<?php

return [

    /*
    |--------------------------------------------------------------------------
    | Языковые строки датчиков
    |--------------------------------------------------------------------------
    |
    | Следующие языковые строки используются для перевода названий и единиц измерения датчиков
    |
    */

    'title' => 'Датчики',
    'airflow' => [
        'short' => 'Воздушный поток',
        'long' => 'Воздушный поток',
        'unit' => 'cfm',
        'unit_long' => 'Кубические футы в минуту',
    ],
    'ber' => [
        'short' => 'BER',
        'long' => 'Коэффициент битовых ошибок',
        'unit' => '',
        'unit_long' => '',
    ],
    'charge' => [
        'short' => 'Заряд',
        'long' => 'Процент заряда',
        'unit' => '%',
        'unit_long' => 'Процент',
    ],
    'chromatic_dispersion' => [
        'short' => 'Хроматическая дисперсия',
        'long' => 'Хроматическая дисперсия',
        'unit' => 'ps/nm',
        'unit_long' => 'Пикосекунды на нанометр',
    ],
    'cooling' => [
        'short' => 'Охлаждение',
        'long' => '',
        'unit' => 'W',
        'unit_long' => 'Ватты',
    ],
    'count' => [
        'short' => 'Количество',
        'long' => 'Количество',
        'unit' => '',
        'unit_long' => '',
    ],
    'current' => [
        'short' => 'Ток',
        'long' => 'Ток',
        'unit' => 'A',
        'unit_long' => 'Амперы',
    ],
    'dbm' => [
        'short' => 'dBm',
        'long' => 'dBm',
        'unit' => 'dBm',
        'unit_long' => 'Децибел-миливатт',
    ],
    'delay' => [
        'short' => 'Задержка',
        'long' => 'Задержка',
        'unit' => 's',
        'unit_long' => 'Секунды',
    ],
    'eer' => [
        'short' => 'EER',
        'long' => 'Коэффициент энергетической эффективности',
        'unit' => '',
        'unit_long' => '',
    ],
    'fanspeed' => [
        'short' => 'Скорость вентилятора',
        'long' => 'Скорость вентилятора',
        'unit' => 'RPM',
        'unit_long' => 'Обороты в минуту',
    ],
    'frequency' => [
        'short' => 'Частота',
        'long' => 'Частота',
        'unit' => 'Hz',
        'unit_long' => 'Герцы',
    ],
    'humidity' => [
        'short' => 'Влажность',
        'long' => 'Процент влажности',
        'unit' => '%',
        'unit_long' => 'Процент',
    ],
    'load' => [
        'short' => 'Нагрузка',
        'long' => 'Процент нагрузки',
        'unit' => '%',
        'unit_long' => 'Процент',
    ],
    'loss' => [
        'short' => 'Процент',
        'long' => 'Процент потерь',
        'unit' => '%',
        'unit_long' => 'Процент',
    ],
    'power' => [
        'short' => 'Мощность',
        'long' => 'Мощность',
        'unit' => 'W',
        'unit_long' => 'Ватты',
    ],
    'power_consumed' => [
        'short' => 'Потребляемая мощность',
        'long' => 'Потребляемая мощность',
        'unit' => 'kWh',
        'unit_long' => 'Киловатт-часа',
    ],
    'power_factor' => [
        'short' => 'Коэффициент мощности',
        'long' => 'Коэффициент мощности',
        'unit' => '',
        'unit_long' => '',
    ],
    'pressure' => [
        'short' => 'Давление',
        'long' => 'Давление',
        'unit' => 'kPa',
        'unit_long' => 'Килопаскали',
    ],
    'quality_factor' => [
        'short' => 'Коэффициент качества',
        'long' => 'Коэффициент качества',
        'unit' => '',
        'unit_long' => '',
    ],
    'runtime' => [
        'short' => 'Время работы',
        'long' => 'Время работы',
        'unit' => 'Min',
        'unit_long' => 'Минуты',
    ],
    'signal' => [
        'short' => 'Сигнал',
        'long' => 'Сигнал',
        'unit' => 'dBm',
        'unit_long' => 'Децибел-миливатт',
    ],
    'tv_signal' => [
        'short' => 'Сигнал',
        'long' => 'ТВ сигнал',
        'unit' => 'dBmV',
        'unit_long' => 'Децибел-миливольт',
    ],
    'bitrate' => [
        'short' => 'Битрейт',
        'long' => 'Битрейт',
        'unit' => 'bps',
        'unit_long' => 'бит в секунду',
    ],
    'snr' => [
        'short' => 'SNR',
        'long' => 'Отношение сигнал/шум',
        'unit' => 'dB',
        'unit_long' => 'Децибелы',
    ],
    'state' => [
        'short' => 'Состояние',
        'long' => 'Состояние',
        'unit' => '',
    ],
    'temperature' => [
        'short' => 'Температура',
        'long' => 'Температура',
        'unit' => '°C',
        'unit_long' => '° Цельсия',
    ],
    'voltage' => [
        'short' => 'Напряжение',
        'long' => 'Напряжение',
        'unit' => 'V',
        'unit_long' => 'Вольты',
    ],
    'waterflow' => [
        'short' => 'Поток воды',
        'long' => 'Поток воды',
        'unit' => 'l/m',
        'unit_long' => 'Литры в минуту',
    ],
    'percent' => [
        'short' => 'Процент',
        'long' => 'Процент',
        'unit' => '%',
        'unit_long' => 'Процент',
    ],
];
