<?php
/**
 * RuleStatusLevel
 *
 * PHP version 5
 *
 * @category Class
 * @package  InfluxDB2
 * @author   OpenAPI Generator team
 * @link     https://openapi-generator.tech
 */

/**
 * InfluxDB OSS API Service
 *
 * The InfluxDB v2 API provides a programmatic interface for all interactions with InfluxDB. Access the InfluxDB API using the `/api/v2/` endpoint.
 *
 * OpenAPI spec version: 2.0.0
 * 
 * Generated by: https://openapi-generator.tech
 * OpenAPI Generator version: 3.3.4
 */

/**
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */

namespace InfluxDB2\Model;
use \InfluxDB2\ObjectSerializer;

/**
 * RuleStatusLevel Class Doc Comment
 *
 * @category Class
 * @description The state to record if check matches a criteria.
 * @package  InfluxDB2
 * @author   OpenAPI Generator team
 * @link     https://openapi-generator.tech
 */
class RuleStatusLevel
{
    /**
     * Possible values of this enum
     */
    const UNKNOWN = 'UNKNOWN';
    const OK = 'OK';
    const INFO = 'INFO';
    const CRIT = 'CRIT';
    const WARN = 'WARN';
    const ANY = 'ANY';
    
    /**
     * Gets allowable values of the enum
     * @return string[]
     */
    public static function getAllowableEnumValues()
    {
        return [
            self::UNKNOWN,
            self::OK,
            self::INFO,
            self::CRIT,
            self::WARN,
            self::ANY,
        ];
    }
}


