<?php

return [
    'alerts' => [
        'title' => '警告',
    ],
    'alertlog' => [
        'title' => '警告历史记录',
    ],
    'alertlog-stats' => [
        'title' => '警告历史统计',
    ],
    'availability-map' => [
        'title' => '可用性地图',
    ],
    'component-status' => [
        'title' => '组件状态',
    ],
    'device-summary-horiz' => [
        'title' => '设备摘要（水平布局）',
    ],
    'device-summary-vert' => [
        'title' => '设备摘要（垂直布局）',
    ],
    'device-types' => [
        'title' => '设备类型',
    ],
    'eventlog' => [
        'title' => '事件日志',
    ],
    'generic-graph' => [
        'title' => '图表',
    ],
    'generic-image' => [
        'title' => '外部图像',
    ],
    'globe' => [
        'title' => '地球地图',
    ],
    'graylog' => [
        'title' => 'Graylog',
    ],
    'notes' => [
        'title' => '笔记',
    ],
    'server-stats' => [
        'title' => '服务器统计信息',
    ],
    'syslog' => [
        'title' => '系统日志',
    ],
    'top-devices' => [
        'title' => '顶级设备',
    ],
    'top-errors' => [
        'title' => '顶级错误',
    ],
    'top-interfaces' => [
        'title' => '顶级接口',
    ],
    'worldmap' => [
        'title' => '世界地图',
    ],
];
