<?php

use App\Facades\DeviceCache;
use App\Models\Eventlog;
use App\Models\Port;
use LibreNMS\Enum\Severity;

/*
 * LibreNMS
 *
 * Copyright (c) 2014 Neil Lathwood <https://github.com/laf/ http://www.lathwood.co.uk/fa>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.  Please see LICENSE.txt at the top level of
 * the source code distribution for details.
*/

Gate::authorize('port.update');

header('Content-type: application/json');

$status = 'error';

$descr = $_POST['descr'];
$device_id = $_POST['device_id'];
$ifName = $_POST['ifName'];
$port_id = $_POST['port_id'];

if (! empty($ifName) && is_numeric($port_id)) {
    // We have ifName and  port id so update ifAlias
    if (empty($descr)) {
        $descr = 'repoll';
        // Set to repoll so we avoid using ifDescr on port poll
    }
    if (Port::where('port_id', $port_id)->update(['ifAlias' => $descr]) > 0) {
        $device = DeviceCache::get($device_id);
        if ($descr === 'repoll') {
            $device->forgetAttrib('ifName:' . $ifName);
            Eventlog::log("$ifName Port ifAlias cleared manually", $device->device_id, 'interface', Severity::Notice, $port_id);
        } else {
            $device->setAttrib('ifName:' . $ifName, 1);
            Eventlog::log("$ifName Port ifAlias set manually: $descr", $device->device_id, 'interface', Severity::Notice, $port_id);
        }
        $status = 'ok';
    } else {
        $status = 'na';
    }
}

$response = [
    'status' => $status,
];
echo json_encode($response, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE);
