<?php

/**
 * geist-watchdog.inc.php
 *
 * LibreNMS temperature poller module for Geist Watchdog
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * @link       https://www.librenms.org
 *
 * @copyright  2017 Neil Lathwood
 * @author     Neil Lathwood <gh+n@laf.io>
 */
$temp_unit = SnmpQuery::cache()->get('GEIST-V4-MIB::temperatureUnits.0')->value();

if ($temp_unit === '0') {
    $sensor_value = fahrenheit_to_celsius($sensor_value / 10) * 10;
}
