<?php

return [

    /*
    |--------------------------------------------------------------------------
    | Sensors Language Lines
    |--------------------------------------------------------------------------
    |
    | The following language lines are used to translate names and units of sensors
    |
    */

    'title' => 'სენსორები',
    'airflow' => [
        'short' => 'ჰაერის დინება',
        'long' => 'ჰაერის დინება',
        'unit' => 'კფ/წთ',
        'unit_long' => 'კუბური ფუტი წუთში',
    ],
    'ber' => [
        'short' => 'ბშს',
        'long' => 'ბიტური შეცდომების სიხშირე',
        'unit' => '',
        'unit_long' => '',
    ],
    'charge' => [
        'short' => 'მუხტი',
        'long' => 'მუხტი პროცენტებში',
        'unit' => '%',
        'unit_long' => 'პროცენტი',
    ],
    'chromatic_dispersion' => [
        'short' => 'ქრომატული დისპერსია',
        'long' => 'ქრომატული დისპერსია',
        'unit' => 'პწმ/ნმ',
        'unit_long' => 'პიკოწამი ნანომეტრში',
    ],
    'cooling' => [
        'short' => 'გაგრილება',
        'long' => '',
        'unit' => 'ვტ',
        'unit_long' => 'ვატი',
    ],
    'count' => [
        'short' => 'რაოდენობა',
        'long' => 'რაოდენობა',
        'unit' => '',
        'unit_long' => '',
    ],
    'current' => [
        'short' => 'დენი',
        'long' => 'დენი',
        'unit' => 'ა',
        'unit_long' => 'ამპერი',
    ],
    'dbm' => [
        'short' => 'დბმ',
        'long' => 'დბმ',
        'unit' => 'დბმ',
        'unit_long' => 'დეციბენ-მილივატი',
    ],
    'delay' => [
        'short' => 'დაყოვნება',
        'long' => 'დაყოვნება',
        'unit' => 'წმ',
        'unit_long' => 'წამი',
    ],
    'eer' => [
        'short' => 'ეეფ',
        'long' => 'ენერგიის ეფექტურობის ფარდობა',
        'unit' => '',
        'unit_long' => '',
    ],
    'fanspeed' => [
        'short' => 'ვენტილ. სიჩქარე',
        'long' => 'ვენტილატორის სიჩქარე',
        'unit' => 'ბრ/წთ',
        'unit_long' => 'ბრუნვა წუთში',
    ],
    'frequency' => [
        'short' => 'სიხშირე',
        'long' => 'სიხშირე',
        'unit' => 'ჰც',
        'unit_long' => 'ჰერცი',
    ],
    'humidity' => [
        'short' => 'ტენიანობა',
        'long' => 'ტენიანობა პროცენტებში',
        'unit' => '%',
        'unit_long' => 'პროცენტი',
    ],
    'load' => [
        'short' => 'დატვირთვა',
        'long' => 'დატვირთვა პროცენტებში',
        'unit' => '%',
        'unit_long' => 'პროცენტი',
    ],
    'loss' => [
        'short' => 'პროცენტი',
        'long' => 'დაკარგული პროცენტებში',
        'unit' => '%',
        'unit_long' => 'პროცენტი',
    ],
    'power' => [
        'short' => 'სიმძლავრე',
        'long' => 'სიმძლავრე',
        'unit' => 'ვტ',
        'unit_long' => 'ვატი',
    ],
    'power_consumed' => [
        'short' => 'ხარჯვადი სიმძლავრე',
        'long' => 'ხარჯვადი სიმძლავრე',
        'unit' => 'კვტ/სთ',
        'unit_long' => 'კილოვატ-საათი',
    ],
    'power_factor' => [
        'short' => 'სიმძლავრის კოეფიციენტი',
        'long' => 'სიმძლავრის კოეფიციენტი',
        'unit' => '',
        'unit_long' => '',
    ],
    'pressure' => [
        'short' => 'წნევა',
        'long' => 'წნევა',
        'unit' => 'კპა',
        'unit_long' => 'კილოპასკალი',
    ],
    'quality_factor' => [
        'short' => 'ხარისხის კოეფიციენტი',
        'long' => 'ხარისხის კოეფიციენტი',
        'unit' => '',
        'unit_long' => '',
    ],
    'runtime' => [
        'short' => 'ჩართულობის დრო',
        'long' => 'ჩართულობის დრო',
        'unit' => 'წთ',
        'unit_long' => 'წუთი',
    ],
    'signal' => [
        'short' => 'სიგნალი',
        'long' => 'სიგნალი',
        'unit' => 'დბმ',
        'unit_long' => 'დეციბელ-მილივატი',
    ],
    'tv_signal' => [
        'short' => 'სიგნალი',
        'long' => 'TV სიგნალი',
        'unit' => 'ბმმვ',
        'unit_long' => 'დეციბელ-მილივოლტი',
    ],
    'toner' => [
        'short' => 'ტონერი',
        'long' => 'ტონერი',
        'unit' => '%',
        'unit_long' => 'პროცენტი',
    ],
    'bitrate' => [
        'short' => 'ბიტური სიჩქარე',
        'long' => 'ბიტური სიჩქარე',
        'unit' => 'ბ/წმ',
        'unit_long' => 'ბიტი წამში',
    ],
    'snr' => [
        'short' => 'SNR',
        'long' => 'სიგნალის ფარდობა ხმაურთან',
        'unit' => 'დბ',
        'unit_long' => 'დეციბელი',
    ],
    'state' => [
        'short' => 'მდგომარეობა',
        'long' => 'მდგომარეობა',
        'unit' => '',
    ],
    'temperature' => [
        'short' => 'ტემპერატურა',
        'long' => 'ტემპერატურა',
        'unit' => '°C',
        'unit_long' => '° ცელსიუსი',
    ],
    'voltage' => [
        'short' => 'ძაბვა',
        'long' => 'ძაბვა',
        'unit' => 'ვ',
        'unit_long' => 'ვოლტი',
    ],
    'waterflow' => [
        'short' => 'წყლის დინება',
        'long' => 'წყლის დინება',
        'unit' => 'ლ/წტ',
        'unit_long' => 'ლიტრი წუთში',
    ],
    'percent' => [
        'short' => 'პროცენტი',
        'long' => 'პროცენტი',
        'unit' => '%',
        'unit_long' => 'პროცენტი',
    ],
];
