<?php

return [
    'oui' => [
        'title' => 'MAC OUI მომწოდებლის ძებნა',
        'lookup' => 'OUI ძებნა',
        'prompt' => 'შეიყვანეთ MAC მისამართ(ებ)-ი მოსაძებნად:',
        'no_db' => 'პროგრამის თქვენს ვერსიას მომწოდებლის მონაცემები არ აქვს. ეს დღიურ პროცესს უნდა გამოეთხოვა. სცადეთ, ხელით გაუშვათ ბრძანება :command .',
        'not_enabled' => 'ამ ფუნქციის გამოსაყენებლად ჩართეთ პარამეტრი :setting .',
    ],
];
