<?php

return [
    'attributes' => [
        'features' => 'OS-ის ფუნქციები',
        'hardware' => 'აპარატურა',
        'icon' => 'ხატულა',
        'ip' => 'IP',
        'location' => 'მდებარეობა',
        'os' => 'მოწყობილობის ოს-ი',
        'serial' => 'სერიული ნომერი',
        'sysName' => 'sysName',
        'version' => 'ოს-ის ვერსია',
        'type' => 'მოწყობილობის ტიპი',
    ],
];
