<?php

namespace Jalle19\CertificateParser\Provider\Exception;

/**
 * Class ProviderException
 * @package Jalle19\CertificateParser\Provider\Exception
 */
class ProviderException extends \Exception
{

}
