<?php

return [

    /*
    |--------------------------------------------------------------------------
    | Password Reset Language Lines
    |--------------------------------------------------------------------------
    |
    | The following language lines are the default lines which match reasons
    | that are given by the password broker for a password update attempt
    | has failed, such as for an invalid token or invalid new password.
    |
    */

    'password' => 'Das Passwort muss aus mindestens acht Zeichen bestehen',
    'reset' => 'Ihr Passwort wurde zurückgesetzt!',
    'sent' => 'Sie haben via E-Mail einen Link bekommen um das Passwort zurückzusetzen!',
    'token' => 'Das Passwort Token zum zurücksetzen ist fehlerhaft.',
    'user' => 'Es konnte kein Benutzer mit der angegebenen E-Mailadresse gefunden werden',

];
