<?php

return [
    'stp_info' => 'Інформація про екземпляр STP',
    'stp_ports' => 'Порти STP',
    'vlan' => 'VLAN',
    'root_bridge' => 'Root bridge',
    'bridge_address' => 'Bridge адреса (MAC)',
    'protocol' => 'Специфікація протокола',
    'priority' => 'Пріоритет',
    'last_topology_change' => 'Час з моменту зміни топології',
    'topology_changes' => 'Зміни топології',
    'designated_root' => 'Призначений root (MAC)',
    'root_cost' => 'Root cost',
    'root_port' => 'Root port',
    'max_age' => 'Максимальний вік (s)',
    'hello_time' => 'Час Hello (s)',
    'hold_time' => 'Час утримання (s)',
    'forward_delay' => 'Затримка перенаправлення (s)',
    'bridge_max_age' => 'Максимальний вік Bridge (s)',
    'bridge_hello_time' => 'Час Hello Bridge (s)',
    'bridge_forward_delay' => 'Затримка перенаправлення Bridge (s)',
];
