<?php

$unit_text = 'Kbytes';

$stats_list = [
    'data-size' => [
        'stat' => 'data-size',
        'descr' => 'Data',
    ],
    'stack-size' => [
        'stat' => 'stack-size',
        'descr' => 'Stack',
    ],
    'text-size' => [
        'stat' => 'text-size',
        'descr' => 'Text',
    ],
];

require 'oslv_monitor-common.inc.php';
