<?php

return [
    'groups' => [
        'updated' => ':port: группы обновлены',
        'none' => ':port не запрашивалось обновление',
    ],
    'filters' => [
        'status_up' => 'Показать только активные',
        'admin_down' => 'Показать отключенные администратором',
        'disabled' => 'Показать отключенные',
        'ignored' => 'Показать игнорируемые',
    ],
    'graphs' => [
        'bits' => 'Биты',
        'upkts' => 'Юникаст-пакеты',
        'nupkts' => 'Нон-юникаст-пакеты',
        'errors' => 'Ошибки',
        'etherlike' => 'Etherlike',
    ],
    'mtu_label' => 'MTU :mtu',
    'tabs' => [
        'arp' => 'ARP таблица',
        'fdb' => 'FDB таблица',
        'links' => 'Соседи',
        'transceivers' => 'Трансиверы',
        'xdsl' => 'xDSL',
    ],
    'transceiver' => 'Трансивер',
    'transceivers' => [
        'fields' => [
            'model' => 'PN: :model',
            'serial' => 'SN: :serial',
            'revision' => 'Rev: :revision',
            'date' => 'Дата: :date',
            'distance' => 'Расстояние: :distance',
            'encoding' => 'Кодирование: :encoding',
            'cable' => 'Кабель: :cable',
            'ddm' => 'DDM: :ddm',
            'wavelength' => 'Длина волны: :wavelength',
            'connector' => 'Разъем: :connector',
            'channels' => 'Каналы: :channels',
        ],
        'metrics' => [
            'power-tx' => 'Tx мощность|Канал :channel Tx мощность',
            'power-rx' => 'Rx мощность|Канал :channel Rx мощность',
            'bias' => 'Смещение|Канал :channel Смещение',
            'temperature' => 'Температура|Канал :channel Температура',
            'voltage' => 'Напряжение|Канал :channel Напряжение',
        ],
        'units' => [
            'power-tx' => 'dBm',
            'power-rx' => 'dBm',
            'bias' => 'mA',
            'temperature' => '°C',
            'voltage' => 'V',
        ],
    ],
    'unknown_port' => 'Неизвестный порт',
    'vlan_count' => 'VLAN: :count',
    'vlan_label' => 'VLAN: :label',
    'vrf_label' => 'VRF: :name',
    'xdsl' => [
        'sync_stat' => 'Синхронизация: :down/:up',
        'attainable_stat' => 'Макс: :down/:up',
        'attenuation_stat' => 'Затухание: :down/:up',
        'snr_stat' => 'SNR: :down/:up',
        'sync' => 'Скорость синхронизации',
        'attainable' => 'Достижимая скорость',
        'attenuation' => 'Затухание',
        'snr' => 'Запас SNR',
        'power' => 'Выходная мощность',
    ],
];
