<?php

return [
    'checks' => [
        'comment' => 'Comentário',
        'item' => 'Item',
        'php_required' => ':version ou superior necessário',
        'status' => 'Status',
        'title' => 'Checagens Pré-Instalação',
    ],
    'database' => [
        'credentials' => 'Credenciais do Banco de Dados',
        'host' => 'Host',
        'host_placeholder' => 'Use localhost para Unix-Socket',
        'name' => 'Nome do Banco de Dados',
        'password' => 'Senha',
        'port' => 'Porta',
        'port_placeholder' => 'Deixe em branco se estiver usando Unix-Socket',
        'socket' => 'Unix-Socket',
        'socket_placeholder' => 'Use apenas para caminho de socket personalizado',
        'test' => 'Verificar as Credenciais',
        'title' => 'Configurar o Banco de Dados',
        'username' => 'Usuário',
    ],
    'finish' => [
        'config_exists' => 'O arquivo config.php já existe',
        'config_not_required' => 'Este arquivo não é necessário. Este é o padrão.',
        'config_not_written' => 'Não foi possível escrever o arquivo config.php',
        'config_written' => 'Arquivo config.php escrito',
        'copied' => 'Copiado para a área de transferência',
        'dashboard' => 'Painel',
        'env_manual' => 'Atualize manualmente :file com o seguinte conteúdo',
        'env_not_written' => 'Não foi possível escrever o arquivo .env',
        'env_written' => 'Arquivo .env escrito',
        'failed' => 'Falha ao salvar arquivo .env',
        'finish' => 'Finalizar Instalação',
        'manual_copy' => 'Pressione Ctrl-C para copiar',
        'retry' => 'Tentar Novamente',
        'settings' => 'Configurações Adicionais',
        'success' => 'Instalação Completa',
        'thanks' => 'Obrigado por usar o LibreNMS.',
        'title' => 'Finalizar a Instalação',
        'validate_button' => 'Validar a Instalação',
    ],
    'install' => 'Instalar',
    'migrate' => [
        'building_interrupt' => 'Não feche esta página ou interrompa a importação!',
        'error' => 'Erro encontrado, verifique a saída para mais detalhes.',
        'migrate' => 'Construir o Banco de Dados',
        'retry' => 'Tentar Novamente',
        'timeout' => 'Solicitação HTTP expirou, a estrutura do seu banco de dados pode estar inconsistente.',
        'wait' => 'Por Favor, aguarde...',
    ],
    'steps' => [
        'checks' => 'Checagens Pré-Instalação',
        'database' => 'Banco de Dados',
        'finish' => 'Finalizar a Instalação',
        'migrate' => 'Construir o Banco de Dados',
        'user' => 'Criar Usuário',
    ],
    'title' => 'Instalação do LibreNMS',
    'user' => [
        'button' => 'Adicionar Usuário',
        'created' => 'Usuário Criado',
        'email' => 'E-mail',
        'failure' => 'Falha ao criar usuário',
        'password' => 'Senha',
        'success' => 'Usuário criado com sucesso',
        'title' => 'Criar Usuário Administrador',
        'username' => 'Nome de Usuário',
    ],
];
