<?php

$graphs = [
    'memcached_bits' => 'Traffic',
    'memcached_commands' => 'Commands',
    'memcached_data' => 'Data Size',
    'memcached_items' => 'Items',
    'memcached_hitmiss' => 'Hits/Misses',
    'memcached_uptime' => 'Uptime',
    'memcached_threads' => 'Threads',

];

foreach ($graphs as $key => $text) {
    $graph_array['height'] = '100';
    $graph_array['width'] = '215';
    $graph_array['to'] = \App\Facades\LibrenmsConfig::get('time.now');
    $graph_array['id'] = $app['app_id'];
    $graph_array['type'] = 'application_' . $key;

    echo '<div class="panel panel-default">
    <div class="panel-heading">
        <h3 class="panel-title">' . $text . '</h3>
    </div>
    <div class="panel-body">
    <div class="row">';
    include 'includes/html/print-graphrow.inc.php';
    echo '</div>';
    echo '</div>';
    echo '</div>';
}
