<?php

/*
 * CustomMapMenuGroupController.php
 *
 * -Description-
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    LibreNMS
 * @link       http://librenms.org
 * @copyright  2024 Tony Murray
 * @author     Tony Murray <murraytony@gmail.com>
 */

namespace App\Http\Controllers\Select;

use App\Models\CustomMap;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Http\Request;

/**
 * @extends SelectController<CustomMap>
 */
class CustomMapMenuGroupController extends SelectController
{
    protected function searchFields($request): array
    {
        return ['menu_group'];
    }

    protected function baseQuery(Request $request): Builder
    {
        $this->authorize('viewAny', CustomMap::class);

        return CustomMap::query()->hasAccess($request->user())
            ->whereNotNull('menu_group')->select('menu_group')->groupBy('menu_group');
    }
}
