<?php

return [
    'alerts' => [
        'title' => 'Оповещения',
    ],
    'alertlog' => [
        'title' => 'История оповещений',
    ],
    'alertlog-stats' => [
        'title' => 'Статистика истории оповещений',
    ],
    'availability-map' => [
        'title' => 'Карта доступности',
    ],
    'component-status' => [
        'title' => 'Статус компонентов',
    ],
    'custom-map' => [
        'title' => 'Пользовательская карта',
    ],
    'device-summary-horiz' => [
        'title' => 'Сводка по устройствам (горизонтально)',
    ],
    'device-summary-vert' => [
        'title' => 'Сводка по устройствам (вертикально)',
    ],
    'device-types' => [
        'title' => 'Типы устройств',
    ],
    'eventlog' => [
        'title' => 'Журнал событий',
    ],
    'generic-graph' => [
        'title' => 'График',
    ],
    'generic-image' => [
        'title' => 'Внешние изображения',
    ],
    'globe' => [
        'title' => 'Глобус',
    ],
    'graylog' => [
        'title' => 'Graylog',
    ],
    'notes' => [
        'title' => 'Заметки',
    ],
    'server-stats' => [
        'title' => 'Статистика сервера',
    ],
    'syslog' => [
        'title' => 'Syslog',
    ],
    'top-devices' => [
        'title' => 'Топ устройств',
    ],
    'top-errors' => [
        'title' => 'Топ ошибок',
    ],
    'top-interfaces' => [
        'title' => 'Топ интерфейсов',
    ],
    'worldmap' => [
        'title' => 'Мировая карта',
    ],
];
