<?php

return [

    /*
    |--------------------------------------------------------------------------
    | Password Reset Language Lines
    |--------------------------------------------------------------------------
    |
    | The following language lines are the default lines which match reasons
    | that are given by the password broker for a password update attempt
    | has failed, such as for an invalid token or invalid new password.
    |
    */

    'reset' => 'Ваш пароль було скинуто!',
    'sent' => 'Ми надіслали посилання на скид паролю на вашу адресу електронної пошти!',
    'throttled' => 'зачекайте перед повторним вводом.',
    'token' => 'Цей токен скиду паролю не валідний.',
    'user' => 'Ми не можемо знайти користувача з такою адресою електронної пошти.',

];
