<?php

return [
    'checks' => [
        'comment' => 'კომენტარი',
        'item' => 'ელემენტი',
        'php_required' => 'აუცილებელია :version ან უფრო მაღალი',
        'status' => 'სტატუსი',
        'title' => 'დაყენებისწინა შემოწმებები',
    ],
    'database' => [
        'credentials' => 'მონაცემთა ბაზის ავტორიზაციის დეტალები',
        'host' => 'ჰოსტი',
        'host_placeholder' => 'Unix-ის სოკეტისთვის გამოიყენეთ localhost',
        'name' => 'მონაცემთა ბაზის სახელი',
        'password' => 'პაროლი',
        'port' => 'პორტი',
        'port_placeholder' => 'დატოვეთ ცარიელი Unux-ის სოკეტის გამოსაყენებლად',
        'socket' => 'Unix-ის სოკეტი',
        'socket_placeholder' => 'ეს, მხოლოდ, სოკეტის ბილიკის ხელით მისათითებლადაა საჭირო',
        'test' => 'ავტორიზაციის დეტალების შემოწმება',
        'title' => 'მონაცემთა ბაზის მორგება',
        'username' => 'მომხმარებელი',
    ],
    'finish' => [
        'config_exists' => 'ფაილი config.php უკვე არსებობს',
        'config_not_required' => 'ეს ფაილი აუცილებელი არაა. ეს ნაგულისხმევია.',
        'config_not_written' => 'config.php-ში ვერ ჩავწერე',
        'config_written' => 'ფაილი config.php ჩაწერილია',
        'copied' => 'დაკოპირდა ბუფერში',
        'dashboard' => 'სამუშაო მაგიდა',
        'env_manual' => 'ხელით განაახლეთ ფაილი :file შემდეგი შემცველობით',
        'env_not_written' => 'ვერ ჩავწერე ფაილი .env',
        'env_written' => 'ფაილი .env ჩაიწერა',
        'failed' => '.env-ში ჩაწერა ჩავარდა',
        'finish' => 'დაყენების დასრულება',
        'manual_copy' => 'Ctrl-C დასაკოპირებლად',
        'retry' => 'თავიდან ცდა',
        'settings' => 'დამატებითი პარამეტრები',
        'success' => 'დაყენება დასრულდა',
        'thanks' => 'მადლობა, რომ  LibreNMS დააყენეთ.',
        'title' => 'დაყენების დასრულება',
        'validate_button' => 'დაყენების გადამოწმება',
    ],
    'install' => 'დაყენება',
    'migrate' => [
        'building_interrupt' => 'არ დახუროთ ეს გვერდი, ან შეწყვიტოთ შემოტანის პროცესი!',
        'error' => 'აღმოჩენილია შეცდომა. დეტალებისთვის იხილეთ გამოტანილი ინფორმაცია.',
        'migrate' => 'მონაცემთა ბაზის თავიდან აგება',
        'retry' => 'თავიდან ცდა',
        'timeout' => 'HTTP მოთხოვნის მოლოდინის ვადა ამოიწურა. შეიძლება, თქვენი მონაცემთა ბაზის სტრუქტურა არამდგრადია.',
        'wait' => 'მოითმინეთ...',
    ],
    'steps' => [
        'checks' => 'დაყენებისწინა შემოწმებები',
        'database' => 'მონაცემთა ბაზა',
        'finish' => 'დაყენების დასრულება',
        'migrate' => 'მონაცემთა ბაზის აგება',
        'user' => 'მომხმარებლის შექმნა',
    ],
    'title' => 'LibreNMS-ს დაყენება',
    'user' => [
        'button' => 'მომხმარებლის დამატება',
        'created' => 'მომხმარებელი შეიქმნა',
        'email' => 'ელფოსტა',
        'failure' => 'მომხმარებლის შექმნა ჩავარდა',
        'password' => 'პაროლი',
        'success' => 'მომხმარებელი წარმატებით შეიქმნა',
        'title' => 'ადმინისტრატორი მოხმარებლის შექმნა',
        'username' => 'მომხმარებლის სახელი',
    ],
];
