<?php

return [
    'settings_page' => ':plugin:-ის მორგება',
    'admin_page' => 'დამატებების ადმინი',
    'admin_title' => 'სისტემური დამატებები',
    'errors' => [
        'not_exist' => 'დამატება :plugin არ არსებობს.',
        'disabled' => 'დამატება :plugin გამორთულია.',
        'view_missing' => 'ნაკლული ხედი.',
    ],
];
