<?php

return [

    /*
    |--------------------------------------------------------------------------
    | Authentication Language Lines
    |--------------------------------------------------------------------------
    |
    | The following language lines are used during authentication for various
    | messages that we need to display to the user. You are free to modify
    | these language lines according to your application's requirements.
    |
    */

    'title' => '驗證',
    'failed' => '這些憑證與我們的記錄不符。',
    'throttle' => '嘗試登入次數過多。請稍候 :seconds 秒再試。',
    'disabled' => '您的帳號已被停用，請聯絡管理員。',
];
