<?php

return [

    /*
    |--------------------------------------------------------------------------
    | Строки языка аутентификации
    |--------------------------------------------------------------------------
    |
    | Следующие языковые строки используются во время аутентификации для различных
    | сообщений, которые мы должны отображать пользователю. Вы можете свободно изменять
    | эти языковые строки в соответствии с требованиями вашего приложения.
    |
    */

    'title' => 'Аутентификация',
    'failed' => 'Эти учетные данные не соответствуют нашим записям.',
    'password' => 'Предоставленный пароль неверен.',
    'throttle' => 'Слишком много попыток входа. Пожалуйста, попробуйте снова через :seconds секунд.',
    'disabled' => 'Ваш аккаунт отключен, пожалуйста, свяжитесь с администратором.',
];
