<?php

return [
    'custom' => [
        'title' => [
            'edit_dialog' => 'Wähle eine Karte zum Bearbeiten',
            'create' => 'Erstelle eine individuelle Karte',
            'view' => ':name | Individuelle Karte',
            'edit' => 'Bearbeite eine individuelle Karte',
            'manage' => 'Verwalte individuelle Karten',
        ],
        'create_map' => 'Neue Karte',
        'view' => [
            'loading' => 'Lade Daten',
            'no_devices' => 'Keine Geräte gefunden',
            'legend' => 'Legende',
            'invalid_link' => 'Link ist down oder Linkgeschwindigkeit ist nicht definiert',
            'go_to' => 'Gehe zu',
            'unknown' => '???',
        ],
        'nodeimage' => [
            'delete' => 'Lösche Bild',
            'upload' => 'Lade Bild hoch',
            'name' => 'Bildname',
            'new' => 'Neues Bild',
            'title' => [
                'edit' => 'Bearbeite Knotenbild',
                'manage' => 'Verwalte Knotenbilder',
                'new' => 'Neues Knotenbild',
            ],
        ],
        'edit' => [
            'text_font' => 'Schriftart',
            'text_size' => 'Schriftgröße',
            'text_color' => 'Schriftfarbe',
            'defaults' => 'Standardwerte festlegen',
            'bg' => [
                'title' => 'Hintergrund festlegen',
                'background' => 'Hintergrund',
                'color' => 'Farbe',
                'image' => 'Bild',
                'map' => 'Karte',
                'none' => 'Keine',
                'clear_bg' => 'Hintergrund löschen',
                'clear_background' => 'Hintergrund löschen',
                'keep_background' => 'Hintergrund behalten',
                'saving' => 'Speichern...',
                'save_errors' => 'Speichern fehlgeschlagen aufgrund der folgenden Fehler:',
                'save_error' => 'Speichern fehlgeschlagen: Der Server meldet folgenden Fehlercode :code',
                'save' => 'Hintergrund speichern',
                'lat' => 'Breitengrad',
                'lng' => 'Längengrad',
                'zoom' => 'Zoom',
                'adjust_map' => 'Karte anpassen',
                'adjust_map_finish' => 'Kartenanpassung beenden',
                'as_image' => 'Als Bild festlegen',
                'as_image_hint' => 'Das Festlegen der Karte als Bildhintergrund macht sie statisch, verbessert jedoch die Leistung und ermöglicht die Verwendung ohne Verbindung zum Kartenkachelserver',
            ],
            'map' => [
                'settings_title' => 'Karteneinstellungen',
                'selectall' => 'Alles auswählen',
                'name' => 'Name',
                'menu_group' => 'Menügruppe',
                'multiselect_info' => 'Entweder lang klicken oder Strg gedrückt halten, um mehrere Knoten auszuwählen',
                'no_group' => 'Keine Gruppe',
                'width' => 'Breite',
                'height' => 'Höhe',
                'alignment' => 'Knotenausrichtung',
                'edgeseparation' => 'Linktrennung',
                'reverse' => 'Pfeile umkehren',
                'saving' => 'Speichern...',
                'save_errors' => 'Speichern fehlgeschlagen aufgrund der folgenden Fehler:',
                'save_error' => 'Speichern fehlgeschlagen: Der Server meldet folgenden Fehlercode :code',
                'clone' => ':name klonen?',
                'delete' => ':name löschen?',
                'list' => 'Zurück zur Kartenliste',
                'unsavedchanges' => 'Es gibt ungespeicherte Änderungen. Bestätigen drücken, um die Änderungen zu verwerfen und zur Kartenliste zurückzukehren, oder Abbrechen, um zum Editor zurückzukehren.',
                'edit' => 'Karteneinstellungen bearbeiten',
                'rerender' => 'Karte neu rendern',
                'save' => 'Karte speichern',
                'legend' => [
                    'customcolours' => 'Benutzerdefinierte Farben',
                    'colour' => 'Farbe',
                    'colour_lower_pct' => 'Startprozent',
                    'colour_down' => 'Farbe bei Geräteausfall',
                    'colour_invalid' => 'Farbe bei unbekanntem Prozentsatz',
                    'font_size' => 'Legendenschriftgröße',
                    'steps' => 'Legendenstufen',
                    'hideinvalid' => 'Ungültige ausblenden',
                    'hideoverspeed' => '100%+ ausblenden',
                ],
                'legend_title' => 'Legendeneinstellungen',
                'legend_toggle' => 'Legende ein-/ausblenden',
                'zoom' => 'Verschieben und Zoomen',
                'dragnodes' => 'Knoten verschieben',
                'physics' => 'Physik-Engine',
            ],
            'node' => [
                'new' => 'Neuer Knoten',
                'add' => 'Knoten hinzufügen',
                'edit' => 'Knoten bearbeiten',
                'defaults_title' => 'Knoten-Standardkonfiguration',
                'label' => 'Bezeichnung',
                'name' => 'Knotenname',
                'device_select' => 'Gerät auswählen',
                'edit_defaults' => 'Knoten-Standardwerte bearbeiten',
                'map_link' => 'Link zur Karte',
                'map_select' => 'Karte auswählen...',
                'style' => 'Stil',
                'style_options' => [
                    'box' => 'Rechteck',
                    'circle' => 'Kreis',
                    'database' => 'Datenbank',
                    'ellipse' => 'Ellipse',
                    'text' => 'Text',
                    'device_image' => 'Gerätebild',
                    'device_image_circle' => 'Gerätebild (rund)',
                    'diamond' => 'Raute',
                    'dot' => 'Punkt',
                    'star' => 'Stern',
                    'triangle' => 'Dreieck',
                    'triangle_inverted' => 'Dreieck umgekehrt',
                    'hexagon' => 'Sechseck',
                    'square' => 'Quadrat',
                    'icon' => 'Symbol (unten auswählen)',
                ],
                'icon' => 'Symbol',
                'icon_options' => [
                    'server' => 'Server',
                    'desktop' => 'Desktop',
                    'dish' => 'Satellitenschüssel',
                    'satellite' => 'Satellit',
                    'wifi' => 'WLAN',
                    'cloud' => 'Cloud',
                    'globe' => 'Globus',
                    'tower' => 'Turm',
                    'arrow_right' => 'Pfeil - Rechts',
                    'arrow_left' => 'Pfeil - Links',
                    'arrow_up' => 'Pfeil - Oben',
                    'arrow_down' => 'Pfeil - Unten',
                ],
                'image' => 'Bild',
                'image_options' => [
                    'adc' => 'Application Delivery Controller',
                    'firewall' => 'Firewall',
                    'gtm' => 'Global Traffic Manager',
                    'router' => 'Router',
                    'switch-l2' => 'Switch - L2',
                    'switch-l3' => 'Switch - L3',
                ],
                'size' => 'Knotengröße',
                'bg_color' => 'Hintergrundfarbe',
                'border_color' => 'Rahmenfarbe',
            ],
            'edge' => [
                'new' => 'Neue Verbindung',
                'add' => 'Verbindung hinzufügen',
                'edit' => 'Verbindung bearbeiten',
                'defaults_title' => 'Verbindungs-Standardkonfiguration',
                'dynamic_width' => 'Dynamische Breite',
                'fixed_width' => 'Feste Breite',
                'from' => 'Von',
                'to' => 'Nach',
                'port_select' => 'Port auswählen',
                'reverse' => 'Portrichtung umkehren',
                'edit_defaults' => 'Verbindungs-Standardwerte bearbeiten',
                'style' => 'Linienstil',
                'style_options' => [
                    'dynamic' => 'Dynamisch',
                    'continuous' => 'Durchgehend',
                    'discrete' => 'Diskret',
                    'diagonalCross' => 'Diagonalkreuz',
                    'straightCross' => 'Geradkreuz',
                    'horizontal' => 'Horizontal',
                    'vertical' => 'Vertikal',
                    'curvedCW' => 'Gebogen im Uhrzeigersinn',
                    'curvedCCW' => 'Gebogen gegen Uhrzeigersinn',
                    'cubicBezier' => 'Kubischer Bezier',
                ],
                'show_usage_percent' => 'Prozentuale Auslastung anzeigen',
                'show_usage_bps' => 'Auslastung in bps anzeigen',
                'text_align' => 'Textausrichtung',
                'align_options' => [
                    'horizontal' => 'Horizontal',
                    'top' => 'Oben',
                    'middle' => 'Mitte',
                    'bottom' => 'Unten',
                ],
                'label' => 'Bezeichnung',
                'recenter' => 'Linie zentrieren',
            ],
            'validate' => [
                'width_format' => 'Breite muss eine Zahl gefolgt von px oder % sein',
                'width_percent' => 'Breite in Prozent muss zwischen 10 und 100 liegen',
                'width_pixels' => 'Breite in Pixeln muss mindestens 200 betragen',
                'height_format' => 'Höhe muss eine Zahl gefolgt von px oder % sein',
                'height_percent' => 'Höhe in Prozent muss zwischen 10 und 100 liegen',
                'height_pixels' => 'Höhe in Pixeln muss mindestens 200 betragen',
            ],
        ],
        'widget' => [
            'not_found' => 'Keine Karte ausgewählt. Widget bearbeiten, um eine Karte auszuwählen.',
        ],
    ],
];
