<?php

$unit_text = 'blocks/sec';

$stats_list = [
    'read-blocks' => [
        'stat' => 'read-blocks',
        'descr' => 'Read',
    ],
    'written-blocks' => [
        'stat' => 'written-blocks',
        'descr' => 'Written',
    ],
];

require 'oslv_monitor-common.inc.php';
