<?php

$unit_text = 'count';

$stats_list = [
    'QUEUE' => [
        'stat' => 'QUEUE',
        'descr' => 'QUEUE',
    ],
    'BUILT' => [
        'stat' => 'BUILT',
        'descr' => 'BUILT',
    ],
    'FAIL' => [
        'stat' => 'FAIL',
        'descr' => 'FAIL',
    ],
    'SKIP' => [
        'stat' => 'SKIP',
        'descr' => 'SKIP',
    ],
    'IGNORE' => [
        'stat' => 'IGNORE',
        'descr' => 'IGNORE',
    ],
    'REMAIN' => [
        'stat' => 'REMAIN',
        'descr' => 'REMAIN',
    ],
    'FETCH' => [
        'stat' => 'FETCH',
        'descr' => 'FETCH',
    ],
];

require 'poudriere-common.inc.php';
