<?php

return [
    'custom' => [
        'title' => [
            'edit_dialog' => 'Выберите пользовательскую карту для редактирования',
            'create' => 'Создать новую пользовательскую карту',
            'view' => ':name | Пользовательская карта',
            'edit' => 'Редактировать пользовательскую карту',
            'manage' => 'Управление пользовательскими картами',
        ],
        'create_map' => 'Новая карта',
        'view' => [
            'loading' => 'Загрузка данных',
            'no_devices' => 'Устройства не найдены',
        ],
        'nodeimage' => [
            'delete' => 'Удалить изображение',
            'upload' => 'Загрузить изображение',
            'name' => 'Название изображения',
            'new' => 'Новое изображение',
            'title' => [
                'edit' => 'Редактировать изображение узла',
                'manage' => 'Управление изображениями узлов',
                'new' => 'Новое изображение узла',
            ],
        ],
        'edit' => [
            'text_font' => 'Шрифт текста',
            'text_size' => 'Размер текста',
            'text_color' => 'Цвет текста',
            'defaults' => 'Установить значения по умолчанию',
            'bg' => [
                'title' => 'Установить фон',
                'background' => 'Фон',
                'color' => 'Цвет',
                'image' => 'Изображение',
                'map' => 'Карта',
                'none' => 'Нет',
                'clear_bg' => 'Очистить фон',
                'clear_background' => 'Очистить фон',
                'keep_background' => 'Сохранить фон',
                'saving' => 'Сохранение...',
                'save_errors' => 'Не удалось сохранить из-за следующих ошибок:',
                'save_error' => 'Не удалось сохранить. Сервер вернул код ошибки: :code',
                'save' => 'Сохранить фон',
                'lat' => 'Широта',
                'lng' => 'Долгота',
                'zoom' => 'Масштаб',
                'adjust_map' => 'Настроить карту',
                'adjust_map_finish' => 'Настройка карты завершена',
                'as_image' => 'Установить как изображение',
                'as_image_hint' => 'Установка карты в качестве фона изображения будет статичной, но обеспечит улучшенную производительность и будет работать без подключения к серверу тайлов карты',
            ],
            'map' => [
                'settings_title' => 'Настройки карты',
                'selectall' => 'Выбрать все',
                'name' => 'Название',
                'menu_group' => 'Группа меню',
                'multiselect_info' => 'Долгое нажатие или удерживание ctrl для выбора нескольких узлов',
                'no_group' => 'Без группы',
                'width' => 'Ширина',
                'height' => 'Высота',
                'alignment' => 'Выравнивание узлов',
                'edgeseparation' => 'Разделение связей',
                'reverse' => 'Обратные стрелки',
                'saving' => 'Сохранение...',
                'save_errors' => 'Не удалось сохранить из-за следующих ошибок:',
                'save_error' => 'Не удалось сохранить. Сервер вернул код ошибки: :code',
                'delete' => 'Удалить :name?',
                'list' => 'Вернуться к списку карт',
                'unsavedchanges' => 'У вас есть несохраненные изменения. Нажмите подтвердить, чтобы отказаться от изменений и вернуться к списку карт, или отмените, чтобы вернуться к редактору.',
                'edit' => 'Редактировать настройки карты',
                'rerender' => 'Перерисовать карту',
                'save' => 'Сохранить карту',
                'legend' => [
                    'customcolours' => 'Пользовательские цвета',
                    'colour' => 'Цвет',
                    'colour_lower_pct' => 'Начальный процент',
                    'colour_down' => 'Цвет устройства "Выключено"',
                    'colour_invalid' => 'Цвет неизвестного процента',
                    'font_size' => 'Размер текста легенды',
                    'steps' => 'Шаги легенды',
                    'hideinvalid' => 'Скрыть недействительные',
                    'hideoverspeed' => 'Скрыть 100%+',
                ],
                'legend_title' => 'Настройки легенды',
                'legend_toggle' => 'Переключить легенду',
                'zoom' => 'Панорамирование и масштабирование',
                'dragnodes' => 'Перемещение узлов',
                'physics' => 'Физический движок',
            ],
            'node' => [
                'new' => 'Новый узел',
                'add' => 'Добавить узел',
                'edit' => 'Редактировать узел',
                'defaults_title' => 'Конфигурация узла по умолчанию',
                'label' => 'Метка',
                'name' => 'Имя узла',
                'device_select' => 'Выбрать устройство',
                'edit_defaults' => 'Редактировать значения по умолчанию узла',
                'map_link' => 'Ссылка на карту',
                'map_select' => 'Выбрать карту...',
                'style' => 'Стиль',
                'style_options' => [
                    'box' => 'Квадрат',
                    'circle' => 'Круг',
                    'database' => 'База данных',
                    'ellipse' => 'Эллипс',
                    'text' => 'Текст',
                    'device_image' => 'Изображение устройства',
                    'device_image_circle' => 'Изображение устройства (круглое)',
                    'diamond' => 'Ромб',
                    'dot' => 'Точка',
                    'star' => 'Звезда',
                    'triangle' => 'Треугольник',
                    'triangle_inverted' => 'Перевернутый треугольник',
                    'hexagon' => 'Шестиугольник',
                    'square' => 'Квадрат',
                    'icon' => 'Иконка (выберите ниже)',
                ],
                'icon' => 'Иконка',
                'icon_options' => [
                    'server' => 'Сервер',
                    'desktop' => 'Настольный компьютер',
                    'dish' => 'Спутниковая антенна',
                    'satellite' => 'Спутник',
                    'wifi' => 'Wi-Fi',
                    'cloud' => 'Облако',
                    'globe' => 'Глобус',
                    'tower' => 'Башня',
                    'arrow_right' => 'Стрелка - вправо',
                    'arrow_left' => 'Стрелка - влево',
                    'arrow_up' => 'Стрелка - вверх',
                    'arrow_down' => 'Стрелка - вниз',
                ],
                'image' => 'Изображение',
                'image_options' => [
                    'adc' => 'Контроллер доставки приложений',
                    'firewall' => 'Межсетевой экран',
                    'gtm' => 'Глобальный менеджер трафика',
                    'router' => 'Маршрутизатор',
                    'switch-l2' => 'Коммутатор - L2',
                    'switch-l3' => 'Коммутатор - L3',
                ],
                'size' => 'Размер узла',
                'bg_color' => 'Цвет фона',
                'border_color' => 'Цвет границы',
            ],
            'edge' => [
                'new' => 'Новый край',
                'add' => 'Добавить край',
                'edit' => 'Редактировать край',
                'defaults_title' => 'Конфигурация края по умолчанию',
                'dynamic_width' => 'Динамическая ширина',
                'fixed_width' => 'Фиксированная ширина',
                'from' => 'От',
                'to' => 'К',
                'port_select' => 'Выбрать порт',
                'reverse' => 'Обратное направление порта',
                'edit_defaults' => 'Редактировать значения по умолчанию края',
                'style' => 'Стиль линии',
                'style_options' => [
                    'dynamic' => 'Динамический',
                    'continuous' => 'Непрерывный',
                    'discrete' => 'Дискретный',
                    'diagonalCross' => 'Диагональный крест',
                    'straightCross' => 'Прямой крест',
                    'horizontal' => 'Горизонтальный',
                    'vertical' => 'Вертикальный',
                    'curvedCW' => 'Изогнутый по часовой стрелке',
                    'curvedCCW' => 'Изогнутый против часовой стрелки',
                    'cubicBezier' => 'Кубический Безье',
                ],
                'show_usage_percent' => 'Показать процент использования',
                'show_usage_bps' => 'Показать использование в битах в секунду',
                'text_align' => 'Выравнивание текста',
                'align_options' => [
                    'horizontal' => 'Горизонтальное',
                    'top' => 'Вверх',
                    'middle' => 'Посередине',
                    'bottom' => 'Вниз',
                ],
                'label' => 'Метка',
                'recenter' => 'Центрировать линию',
            ],
            'validate' => [
                'width_format' => 'Ширина должна быть числом, за которым следует px или %',
                'width_percent' => 'Процент ширины должен быть от 10 до 100',
                'width_pixels' => 'Ширина в пикселях должна быть не менее 200',
                'height_format' => 'Высота должна быть числом, за которым следует px или %',
                'height_percent' => 'Процент высоты должен быть от 10 до 100',
                'height_pixels' => 'Высота в пикселях должна быть не менее 200',
            ],
        ],
        'widget' => [
            'not_found' => 'Карта не выбрана. Отредактируйте виджет, чтобы выбрать карту.',
        ],
    ],
];
