<div class="panel panel-default">
    <div class="panel-body ">
        <div class="pull-left" style="margin-top: 5px;">
            <span style="font-size: 20px;">{{ $title }}</a></span><br>
            Description
    </div>
    </div>
</div>
