<?php

namespace Jalle19\CertificateParser\Provider\Exception;

/**
 * Class DomainMismatchException
 * @package Jalle19\CertificateParser\Provider\Exception
 */
class DomainMismatchException extends ProviderException 
{

}
