<?php

return [
    'settings_page' => ':plugin: Настройки',
    'admin_page' => 'Администрирование плагина',
    'admin_title' => 'Системные плагины',
    'errors' => [
        'not_exist' => 'Плагин :plugin не существует.',
        'disabled' => 'Плагин :plugin отключен.',
        'view_missing' => 'Отсутствует представление.',
    ],
];
