<?php

$unit_text = 'Bytes';

$stats_list = [
    'bytes_mean' => [
        'stat' => 'bytes_mean',
        'descr' => 'Mean',
    ],
    'bytes_median' => [
        'stat' => 'bytes_median',
        'descr' => 'Median',
    ],
    'bytes_mode' => [
        'stat' => 'bytes_mode',
        'descr' => 'Mode',
    ],
    'bytes_max' => [
        'stat' => 'bytes_max',
        'descr' => 'Max',
    ],
    'bytes_min' => [
        'stat' => 'bytes_min',
        'descr' => 'Min',
    ],
];

require 'http_access_log_combined-common.inc.php';
