<?php return array(
    'root' => array(
        'name' => 'librenms/librenms',
        'pretty_version' => '1.0.0+no-version-set',
        'version' => '1.0.0.0',
        'reference' => null,
        'type' => 'project',
        'install_path' => __DIR__ . '/../../',
        'aliases' => array(),
        'dev' => false,
    ),
    'versions' => array(
        'acmephp/ssl' => array(
            'pretty_version' => '2.1.0',
            'version' => '2.1.0.0',
            'reference' => 'c78ad953d3680acf3f02024c93ff32724d353590',
            'type' => 'library',
            'install_path' => __DIR__ . '/../acmephp/ssl',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'amenadiel/jpgraph' => array(
            'pretty_version' => 'v4.1.1',
            'version' => '4.1.1.0',
            'reference' => 'e196d51a520a4182cc26240b4a0744495706f11a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../amenadiel/jpgraph',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'brick/math' => array(
            'pretty_version' => '0.14.8',
            'version' => '0.14.8.0',
            'reference' => '63422359a44b7f06cae63c3b429b59e8efcc0629',
            'type' => 'library',
            'install_path' => __DIR__ . '/../brick/math',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'carbonphp/carbon-doctrine-types' => array(
            'pretty_version' => '3.2.0',
            'version' => '3.2.0.0',
            'reference' => '18ba5ddfec8976260ead6e866180bd5d2f71aa1d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../carbonphp/carbon-doctrine-types',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/socket-raw' => array(
            'pretty_version' => 'v1.6.0',
            'version' => '1.6.0.0',
            'reference' => '91e9f619f6769f931454a9882c21ffd7623d06cb',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/socket-raw',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/stream-filter' => array(
            'pretty_version' => 'v1.7.0',
            'version' => '1.7.0.0',
            'reference' => '049509fef80032cb3f051595029ab75b49a3c2f7',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/stream-filter',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'dapphp/radius' => array(
            'pretty_version' => '3.1.0',
            'version' => '3.1.0.0',
            'reference' => 'f2f7b5f062c001beb76bc326946d0d3ee78cb693',
            'type' => 'library',
            'install_path' => __DIR__ . '/../dapphp/radius',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'dflydev/dot-access-data' => array(
            'pretty_version' => 'v3.0.3',
            'version' => '3.0.3.0',
            'reference' => 'a23a2bf4f31d3518f3ecb38660c95715dfead60f',
            'type' => 'library',
            'install_path' => __DIR__ . '/../dflydev/dot-access-data',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'doctrine/inflector' => array(
            'pretty_version' => '2.1.0',
            'version' => '2.1.0.0',
            'reference' => '6d6c96277ea252fc1304627204c3d5e6e15faa3b',
            'type' => 'library',
            'install_path' => __DIR__ . '/../doctrine/inflector',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'doctrine/lexer' => array(
            'pretty_version' => '3.0.1',
            'version' => '3.0.1.0',
            'reference' => '31ad66abc0fc9e1a1f2d9bc6a42668d2fbbcd6dd',
            'type' => 'library',
            'install_path' => __DIR__ . '/../doctrine/lexer',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'dragonmantank/cron-expression' => array(
            'pretty_version' => 'v3.6.0',
            'version' => '3.6.0.0',
            'reference' => 'd61a8a9604ec1f8c3d150d09db6ce98b32675013',
            'type' => 'library',
            'install_path' => __DIR__ . '/../dragonmantank/cron-expression',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'easybook/geshi' => array(
            'pretty_version' => 'v1.0.8.19',
            'version' => '1.0.8.19',
            'reference' => 'b4df5fa84a44d4e12eff67263a701eac7e157241',
            'type' => 'library',
            'install_path' => __DIR__ . '/../easybook/geshi',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'egulias/email-validator' => array(
            'pretty_version' => '4.0.4',
            'version' => '4.0.4.0',
            'reference' => 'd42c8731f0624ad6bdc8d3e5e9a4524f68801cfa',
            'type' => 'library',
            'install_path' => __DIR__ . '/../egulias/email-validator',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'enshrined/svg-sanitize' => array(
            'pretty_version' => '0.22.0',
            'version' => '0.22.0.0',
            'reference' => '0afa95ea74be155a7bcd6c6fb60c276c39984500',
            'type' => 'library',
            'install_path' => __DIR__ . '/../enshrined/svg-sanitize',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ezyang/htmlpurifier' => array(
            'pretty_version' => 'v4.19.0',
            'version' => '4.19.0.0',
            'reference' => 'b287d2a16aceffbf6e0295559b39662612b77fcf',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ezyang/htmlpurifier',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'fico7489/laravel-pivot' => array(
            'pretty_version' => '3.0.13',
            'version' => '3.0.13.0',
            'reference' => 'd36bbebb108b7b6828251c4b001b255084061a2f',
            'type' => 'library',
            'install_path' => __DIR__ . '/../fico7489/laravel-pivot',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'firebase/php-jwt' => array(
            'pretty_version' => 'v7.0.5',
            'version' => '7.0.5.0',
            'reference' => '47ad26bab5e7c70ae8a6f08ed25ff83631121380',
            'type' => 'library',
            'install_path' => __DIR__ . '/../firebase/php-jwt',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'fruitcake/php-cors' => array(
            'pretty_version' => 'v1.4.0',
            'version' => '1.4.0.0',
            'reference' => '38aaa6c3fd4c157ffe2a4d10aa8b9b16ba8de379',
            'type' => 'library',
            'install_path' => __DIR__ . '/../fruitcake/php-cors',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'graham-campbell/result-type' => array(
            'pretty_version' => 'v1.1.4',
            'version' => '1.1.4.0',
            'reference' => 'e01f4a821471308ba86aa202fed6698b6b695e3b',
            'type' => 'library',
            'install_path' => __DIR__ . '/../graham-campbell/result-type',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'guzzlehttp/guzzle' => array(
            'pretty_version' => '7.10.0',
            'version' => '7.10.0.0',
            'reference' => 'b51ac707cfa420b7bfd4e4d5e510ba8008e822b4',
            'type' => 'library',
            'install_path' => __DIR__ . '/../guzzlehttp/guzzle',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'guzzlehttp/promises' => array(
            'pretty_version' => '2.3.0',
            'version' => '2.3.0.0',
            'reference' => '481557b130ef3790cf82b713667b43030dc9c957',
            'type' => 'library',
            'install_path' => __DIR__ . '/../guzzlehttp/promises',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'guzzlehttp/psr7' => array(
            'pretty_version' => '2.11.0',
            'version' => '2.11.0.0',
            'reference' => 'bbb5e61349fa5cb822b3e87842b951088b76b81f',
            'type' => 'library',
            'install_path' => __DIR__ . '/../guzzlehttp/psr7',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'guzzlehttp/uri-template' => array(
            'pretty_version' => 'v1.0.5',
            'version' => '1.0.5.0',
            'reference' => '4f4bbd4e7172148801e76e3decc1e559bdee34e1',
            'type' => 'library',
            'install_path' => __DIR__ . '/../guzzlehttp/uri-template',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'illuminate/auth' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/broadcasting' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/bus' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/cache' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/collections' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/concurrency' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/conditionable' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/config' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/console' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/container' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/contracts' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/cookie' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/database' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/encryption' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/events' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/filesystem' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/hashing' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/http' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/json-schema' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/log' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/macroable' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/mail' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/notifications' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/pagination' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/pipeline' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/process' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/queue' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/redis' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/reflection' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/routing' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/session' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/support' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/testing' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/translation' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/validation' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'illuminate/view' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v12.58.0',
            ),
        ),
        'influxdata/influxdb-client-php' => array(
            'pretty_version' => '3.8.0',
            'version' => '3.8.0.0',
            'reference' => '59ac11d63ce030973c79d5b05797813761a0d58e',
            'type' => 'library',
            'install_path' => __DIR__ . '/../influxdata/influxdb-client-php',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'influxdb/influxdb-php' => array(
            'pretty_version' => '1.15.2',
            'version' => '1.15.2.0',
            'reference' => 'd6e59f4f04ab9107574fda69c2cbe36671253d03',
            'type' => 'library',
            'install_path' => __DIR__ . '/../influxdb/influxdb-php',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'jalle19/certificate-parser' => array(
            'pretty_version' => '4.0.0',
            'version' => '4.0.0.0',
            'reference' => '6ea3b4635004bef09020e706736cb94a5cff919c',
            'type' => 'library',
            'install_path' => __DIR__ . '/../jalle19/certificate-parser',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'justinrainbow/json-schema' => array(
            'pretty_version' => '6.8.2',
            'version' => '6.8.2.0',
            'reference' => '2c89ebb95ca9cedc9347f780333f7b25792dcb76',
            'type' => 'library',
            'install_path' => __DIR__ . '/../justinrainbow/json-schema',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'laravel-notification-channels/webpush' => array(
            'pretty_version' => '10.5.0',
            'version' => '10.5.0.0',
            'reference' => '4e7a9b5accce086c989bc061587ad2f51809887b',
            'type' => 'library',
            'install_path' => __DIR__ . '/../laravel-notification-channels/webpush',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'laravel/framework' => array(
            'pretty_version' => 'v12.58.0',
            'version' => '12.58.0.0',
            'reference' => '6172ae1f44ba5d89e111057ee4a4e7c27f5a610d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../laravel/framework',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'laravel/prompts' => array(
            'pretty_version' => 'v0.3.17',
            'version' => '0.3.17.0',
            'reference' => '6a82ac19a28b916ae0885828795dbd4c59d9a818',
            'type' => 'library',
            'install_path' => __DIR__ . '/../laravel/prompts',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'laravel/serializable-closure' => array(
            'pretty_version' => 'v2.0.13',
            'version' => '2.0.13.0',
            'reference' => 'b566ee0dd251f3c4078bed003a7ce015f5ea6dce',
            'type' => 'library',
            'install_path' => __DIR__ . '/../laravel/serializable-closure',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'laravel/socialite' => array(
            'pretty_version' => 'v5.27.0',
            'version' => '5.27.0.0',
            'reference' => '40e0757a75637c7b2dff05d3286b0d8fc25e5c0e',
            'type' => 'library',
            'install_path' => __DIR__ . '/../laravel/socialite',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'laravel/tinker' => array(
            'pretty_version' => 'v3.0.2',
            'version' => '3.0.2.0',
            'reference' => '4faba77764bd33411735936acdf30446d058c78b',
            'type' => 'library',
            'install_path' => __DIR__ . '/../laravel/tinker',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'laravel/ui' => array(
            'pretty_version' => 'v4.6.3',
            'version' => '4.6.3.0',
            'reference' => 'ff27db15416c1ed8ad9848f5692e47595dd5de27',
            'type' => 'library',
            'install_path' => __DIR__ . '/../laravel/ui',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'league/commonmark' => array(
            'pretty_version' => '2.8.2',
            'version' => '2.8.2.0',
            'reference' => '59fb075d2101740c337c7216e3f32b36c204218b',
            'type' => 'library',
            'install_path' => __DIR__ . '/../league/commonmark',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'league/config' => array(
            'pretty_version' => 'v1.2.0',
            'version' => '1.2.0.0',
            'reference' => '754b3604fb2984c71f4af4a9cbe7b57f346ec1f3',
            'type' => 'library',
            'install_path' => __DIR__ . '/../league/config',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'league/flysystem' => array(
            'pretty_version' => '3.33.0',
            'version' => '3.33.0.0',
            'reference' => '570b8871e0ce693764434b29154c54b434905350',
            'type' => 'library',
            'install_path' => __DIR__ . '/../league/flysystem',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'league/flysystem-local' => array(
            'pretty_version' => '3.31.0',
            'version' => '3.31.0.0',
            'reference' => '2f669db18a4c20c755c2bb7d3a7b0b2340488079',
            'type' => 'library',
            'install_path' => __DIR__ . '/../league/flysystem-local',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'league/mime-type-detection' => array(
            'pretty_version' => '1.16.0',
            'version' => '1.16.0.0',
            'reference' => '2d6702ff215bf922936ccc1ad31007edc76451b9',
            'type' => 'library',
            'install_path' => __DIR__ . '/../league/mime-type-detection',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'league/oauth1-client' => array(
            'pretty_version' => 'v1.11.0',
            'version' => '1.11.0.0',
            'reference' => 'f9c94b088837eb1aae1ad7c4f23eb65cc6993055',
            'type' => 'library',
            'install_path' => __DIR__ . '/../league/oauth1-client',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'league/uri' => array(
            'pretty_version' => '7.8.1',
            'version' => '7.8.1.0',
            'reference' => '08cf38e3924d4f56238125547b5720496fac8fd4',
            'type' => 'library',
            'install_path' => __DIR__ . '/../league/uri',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'league/uri-interfaces' => array(
            'pretty_version' => '7.8.1',
            'version' => '7.8.1.0',
            'reference' => '85d5c77c5d6d3af6c54db4a78246364908f3c928',
            'type' => 'library',
            'install_path' => __DIR__ . '/../league/uri-interfaces',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'librenms/laravel-vue-i18n-generator' => array(
            'pretty_version' => 'dev-master',
            'version' => 'dev-master',
            'reference' => '7e72398d2549389b37b0b7ff6a850264fe1eea80',
            'type' => 'library',
            'install_path' => __DIR__ . '/../librenms/laravel-vue-i18n-generator',
            'aliases' => array(
                0 => '9999999-dev',
            ),
            'dev_requirement' => false,
        ),
        'librenms/librenms' => array(
            'pretty_version' => '1.0.0+no-version-set',
            'version' => '1.0.0.0',
            'reference' => null,
            'type' => 'project',
            'install_path' => __DIR__ . '/../../',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'librenms/plugin-interfaces' => array(
            'pretty_version' => '1.0',
            'version' => '1.0.0.0',
            'reference' => '7a196c3e47c5c5eb140f895dde6ceac998cf7b48',
            'type' => 'library',
            'install_path' => __DIR__ . '/../librenms/plugin-interfaces',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'marc-mabe/php-enum' => array(
            'pretty_version' => 'v4.7.2',
            'version' => '4.7.2.0',
            'reference' => 'bb426fcdd65c60fb3638ef741e8782508fda7eef',
            'type' => 'library',
            'install_path' => __DIR__ . '/../marc-mabe/php-enum',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'mews/purifier' => array(
            'pretty_version' => '3.4.4',
            'version' => '3.4.4.0',
            'reference' => 'b2705cc6c832ce7229373418e191d71b6c037841',
            'type' => 'package',
            'install_path' => __DIR__ . '/../mews/purifier',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'mikepultz/netdns2' => array(
            'pretty_version' => 'v1.5.5',
            'version' => '1.5.5.0',
            'reference' => 'ea39ef5a97d5c2b9893a8c35af7b5fd5b0e40bc9',
            'type' => 'library',
            'install_path' => __DIR__ . '/../mikepultz/netdns2',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'minishlink/web-push' => array(
            'pretty_version' => 'v10.0.3',
            'version' => '10.0.3.0',
            'reference' => '547695eb42b062517fc604c85d6f7bb8174d31b0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../minishlink/web-push',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'monolog/monolog' => array(
            'pretty_version' => '3.10.0',
            'version' => '3.10.0.0',
            'reference' => 'b321dd6749f0bf7189444158a3ce785cc16d69b0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../monolog/monolog',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'mtdowling/cron-expression' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '^1.0',
            ),
        ),
        'nesbot/carbon' => array(
            'pretty_version' => '3.11.4',
            'version' => '3.11.4.0',
            'reference' => 'e890471a3494740f7d9326d72ce6a8c559ffee60',
            'type' => 'library',
            'install_path' => __DIR__ . '/../nesbot/carbon',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'nette/schema' => array(
            'pretty_version' => 'v1.3.5',
            'version' => '1.3.5.0',
            'reference' => 'f0ab1a3cda782dbc5da270d28545236aa80c4002',
            'type' => 'library',
            'install_path' => __DIR__ . '/../nette/schema',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'nette/utils' => array(
            'pretty_version' => 'v4.1.3',
            'version' => '4.1.3.0',
            'reference' => 'bb3ea637e3d131d72acc033cfc2746ee893349fe',
            'type' => 'library',
            'install_path' => __DIR__ . '/../nette/utils',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'nikic/php-parser' => array(
            'pretty_version' => 'v5.7.0',
            'version' => '5.7.0.0',
            'reference' => 'dca41cd15c2ac9d055ad70dbfd011130757d1f82',
            'type' => 'library',
            'install_path' => __DIR__ . '/../nikic/php-parser',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'nunomaduro/laravel-console-summary' => array(
            'pretty_version' => 'v1.14.0',
            'version' => '1.14.0.0',
            'reference' => '21c4c5c432bd0db55d747647a7b3a75be71bf67b',
            'type' => 'library',
            'install_path' => __DIR__ . '/../nunomaduro/laravel-console-summary',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'nunomaduro/termwind' => array(
            'pretty_version' => 'v2.4.0',
            'version' => '2.4.0.0',
            'reference' => '712a31b768f5daea284c2169a7d227031001b9a8',
            'type' => 'library',
            'install_path' => __DIR__ . '/../nunomaduro/termwind',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'paragonie/constant_time_encoding' => array(
            'pretty_version' => 'v3.1.3',
            'version' => '3.1.3.0',
            'reference' => 'd5b01a39b3415c2cd581d3bd3a3575c1ebbd8e77',
            'type' => 'library',
            'install_path' => __DIR__ . '/../paragonie/constant_time_encoding',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'paragonie/random_compat' => array(
            'pretty_version' => 'v9.99.100',
            'version' => '9.99.100.0',
            'reference' => '996434e5492cb4c3edcb9168db6fbb1359ef965a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../paragonie/random_compat',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'pear/console_color2' => array(
            'pretty_version' => '0.1.2',
            'version' => '0.1.2.0',
            'reference' => 'cab2b886aa55cc9e23464d1f49edf97bee9f5a63',
            'type' => 'library',
            'install_path' => __DIR__ . '/../pear/console_color2',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'php-amqplib/php-amqplib' => array(
            'pretty_version' => 'v3.7.4',
            'version' => '3.7.4.0',
            'reference' => '381b6f7c600e0e0c7463cdd7f7a1a3bc6268e5fd',
            'type' => 'library',
            'install_path' => __DIR__ . '/../php-amqplib/php-amqplib',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'php-http/async-client-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '*',
            ),
        ),
        'php-http/client-common' => array(
            'pretty_version' => '2.7.3',
            'version' => '2.7.3.0',
            'reference' => 'dcc6de29c90dd74faab55f71b79d89409c4bf0c1',
            'type' => 'library',
            'install_path' => __DIR__ . '/../php-http/client-common',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'php-http/client-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '*',
            ),
        ),
        'php-http/discovery' => array(
            'pretty_version' => '1.20.0',
            'version' => '1.20.0.0',
            'reference' => '82fe4c73ef3363caed49ff8dd1539ba06044910d',
            'type' => 'composer-plugin',
            'install_path' => __DIR__ . '/../php-http/discovery',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'php-http/httplug' => array(
            'pretty_version' => '2.4.1',
            'version' => '2.4.1.0',
            'reference' => '5cad731844891a4c282f3f3e1b582c46839d22f4',
            'type' => 'library',
            'install_path' => __DIR__ . '/../php-http/httplug',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'php-http/message' => array(
            'pretty_version' => '1.16.2',
            'version' => '1.16.2.0',
            'reference' => '06dd5e8562f84e641bf929bfe699ee0f5ce8080a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../php-http/message',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'php-http/message-factory-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'php-http/promise' => array(
            'pretty_version' => '1.3.1',
            'version' => '1.3.1.0',
            'reference' => 'fc85b1fba37c169a69a07ef0d5a8075770cc1f83',
            'type' => 'library',
            'install_path' => __DIR__ . '/../php-http/promise',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'phpmailer/phpmailer' => array(
            'pretty_version' => 'v7.0.2',
            'version' => '7.0.2.0',
            'reference' => 'ebf1655bd5b99b3f97e1a3ec0a69e5f4cd7ea088',
            'type' => 'library',
            'install_path' => __DIR__ . '/../phpmailer/phpmailer',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'phpoption/phpoption' => array(
            'pretty_version' => '1.9.5',
            'version' => '1.9.5.0',
            'reference' => '75365b91986c2405cf5e1e012c5595cd487a98be',
            'type' => 'library',
            'install_path' => __DIR__ . '/../phpoption/phpoption',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'phpseclib/phpseclib' => array(
            'pretty_version' => '3.0.52',
            'version' => '3.0.52.0',
            'reference' => '2adaefc83df2ec548558307690f376dd7d4f4fce',
            'type' => 'library',
            'install_path' => __DIR__ . '/../phpseclib/phpseclib',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'predis/predis' => array(
            'pretty_version' => 'v3.4.2',
            'version' => '3.4.2.0',
            'reference' => '2033429520d8997a7815a2485f56abe6d2d0e075',
            'type' => 'library',
            'install_path' => __DIR__ . '/../predis/predis',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/clock' => array(
            'pretty_version' => '1.0.0',
            'version' => '1.0.0.0',
            'reference' => 'e41a24703d4560fd0acb709162f73b8adfc3aa0d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/clock',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/clock-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'psr/container' => array(
            'pretty_version' => '2.0.2',
            'version' => '2.0.2.0',
            'reference' => 'c71ecc56dfe541dbd90c5360474fbc405f8d5963',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/container',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/container-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.1|2.0',
            ),
        ),
        'psr/event-dispatcher' => array(
            'pretty_version' => '1.0.0',
            'version' => '1.0.0.0',
            'reference' => 'dbefd12671e8a14ec7f180cab83036ed26714bb0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/event-dispatcher',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/event-dispatcher-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'psr/http-client' => array(
            'pretty_version' => '1.0.3',
            'version' => '1.0.3.0',
            'reference' => 'bb5906edc1c324c9a05aa0873d40117941e5fa90',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-client',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/http-client-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '*',
                1 => '1.0',
            ),
        ),
        'psr/http-factory' => array(
            'pretty_version' => '1.1.0',
            'version' => '1.1.0.0',
            'reference' => '2b4765fddfe3b508ac62f829e852b1501d3f6e8a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-factory',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/http-factory-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '*',
                1 => '1.0',
            ),
        ),
        'psr/http-message' => array(
            'pretty_version' => '2.0',
            'version' => '2.0.0.0',
            'reference' => '402d35bcb92c70c026d1a6a9883f06b2ead23d71',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-message',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/http-message-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '*',
                1 => '1.0',
            ),
        ),
        'psr/log' => array(
            'pretty_version' => '3.0.2',
            'version' => '3.0.2.0',
            'reference' => 'f16e1d5863e37f8d8c2a01719f5b34baa2b714d3',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/log',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/log-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0|2.0|3.0',
                1 => '3.0.0',
            ),
        ),
        'psr/simple-cache' => array(
            'pretty_version' => '3.0.0',
            'version' => '3.0.0.0',
            'reference' => '764e0b3939f5ca87cb904f570ef9be2d78a07865',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/simple-cache',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/simple-cache-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0|2.0|3.0',
            ),
        ),
        'psy/psysh' => array(
            'pretty_version' => 'v0.12.22',
            'version' => '0.12.22.0',
            'reference' => '3be75d5b9244936dd4ac62ade2bfb004d13acf0f',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psy/psysh',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ralouphie/getallheaders' => array(
            'pretty_version' => '3.0.3',
            'version' => '3.0.3.0',
            'reference' => '120b605dfeb996808c31b6477290a714d356e822',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ralouphie/getallheaders',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ramsey/collection' => array(
            'pretty_version' => '2.1.1',
            'version' => '2.1.1.0',
            'reference' => '344572933ad0181accbf4ba763e85a0306a8c5e2',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ramsey/collection',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ramsey/uuid' => array(
            'pretty_version' => '4.9.2',
            'version' => '4.9.2.0',
            'reference' => '8429c78ca35a09f27565311b98101e2826affde0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ramsey/uuid',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'rhumsaa/uuid' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '4.9.2',
            ),
        ),
        'socialiteproviders/manager' => array(
            'pretty_version' => '4.9.2',
            'version' => '4.9.2.0',
            'reference' => '35372dc62787e61e91cfec73f45fd5d5ae0f8891',
            'type' => 'library',
            'install_path' => __DIR__ . '/../socialiteproviders/manager',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'spatie/backtrace' => array(
            'pretty_version' => '1.8.2',
            'version' => '1.8.2.0',
            'reference' => '8ffe78be5ed355b5009e3dd989d183433e9a5adc',
            'type' => 'library',
            'install_path' => __DIR__ . '/../spatie/backtrace',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'spatie/error-solutions' => array(
            'pretty_version' => '1.1.3',
            'version' => '1.1.3.0',
            'reference' => 'e495d7178ca524f2dd0fe6a1d99a1e608e1c9936',
            'type' => 'library',
            'install_path' => __DIR__ . '/../spatie/error-solutions',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'spatie/flare-client-php' => array(
            'pretty_version' => '1.11.0',
            'version' => '1.11.0.0',
            'reference' => 'fb3ffb946675dba811fbde9122224db2f84daca9',
            'type' => 'library',
            'install_path' => __DIR__ . '/../spatie/flare-client-php',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'spatie/ignition' => array(
            'pretty_version' => '1.16.0',
            'version' => '1.16.0.0',
            'reference' => 'b59385bb7aa24dae81bcc15850ebecfda7b40838',
            'type' => 'library',
            'install_path' => __DIR__ . '/../spatie/ignition',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'spatie/laravel-ignition' => array(
            'pretty_version' => '2.12.0',
            'version' => '2.12.0.0',
            'reference' => '45b3b6e1e73fc161cba2149972698644b99594ee',
            'type' => 'library',
            'install_path' => __DIR__ . '/../spatie/laravel-ignition',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'spatie/laravel-permission' => array(
            'pretty_version' => '6.25.0',
            'version' => '6.25.0.0',
            'reference' => 'd7d4cb0d58616722f1afc90e0484e4825155b9b3',
            'type' => 'library',
            'install_path' => __DIR__ . '/../spatie/laravel-permission',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'spatie/once' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '*',
            ),
        ),
        'spomky-labs/base64url' => array(
            'pretty_version' => 'v2.0.4',
            'version' => '2.0.4.0',
            'reference' => '7752ce931ec285da4ed1f4c5aa27e45e097be61d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../spomky-labs/base64url',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'spomky-labs/pki-framework' => array(
            'pretty_version' => '1.4.2',
            'version' => '1.4.2.0',
            'reference' => 'aa576cbd07128075bef97ac2f8af9854e67513d8',
            'type' => 'library',
            'install_path' => __DIR__ . '/../spomky-labs/pki-framework',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/clock' => array(
            'pretty_version' => 'v7.4.8',
            'version' => '7.4.8.0',
            'reference' => '674fa3b98e21531dd040e613479f5f6fa8f32111',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/clock',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/console' => array(
            'pretty_version' => 'v7.4.11',
            'version' => '7.4.11.0',
            'reference' => 'ed0107e43ab452aa77ae99e005b95e56b556e075',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/console',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/css-selector' => array(
            'pretty_version' => 'v7.4.9',
            'version' => '7.4.9.0',
            'reference' => 'b75663ed96cf4756e28e3105476f220f92886cc4',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/css-selector',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/deprecation-contracts' => array(
            'pretty_version' => 'v3.7.0',
            'version' => '3.7.0.0',
            'reference' => '50f59d1f3ca46d41ac911f97a78626b6756af35b',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/deprecation-contracts',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/error-handler' => array(
            'pretty_version' => 'v7.4.8',
            'version' => '7.4.8.0',
            'reference' => '8dd79d8af777ee6cba2fd4d98da6ffb839f3c0fa',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/error-handler',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/event-dispatcher' => array(
            'pretty_version' => 'v7.4.9',
            'version' => '7.4.9.0',
            'reference' => 'e4a2e29753c7801f7a8340e066cfa788f3bc8101',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/event-dispatcher',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/event-dispatcher-contracts' => array(
            'pretty_version' => 'v3.7.0',
            'version' => '3.7.0.0',
            'reference' => 'ccba7060602b7fed0b03c85bf025257f76d9ef32',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/event-dispatcher-contracts',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/event-dispatcher-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '2.0|3.0',
            ),
        ),
        'symfony/finder' => array(
            'pretty_version' => 'v7.4.8',
            'version' => '7.4.8.0',
            'reference' => 'e0be088d22278583a82da281886e8c3592fbf149',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/finder',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/http-foundation' => array(
            'pretty_version' => 'v7.4.13',
            'version' => '7.4.13.0',
            'reference' => 'bc354f47c62301e990b7874fa662326368508e2c',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/http-foundation',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/http-kernel' => array(
            'pretty_version' => 'v7.4.13',
            'version' => '7.4.13.0',
            'reference' => '9df847980c436451f4f51d1284491bb4356dd989',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/http-kernel',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/mailer' => array(
            'pretty_version' => 'v7.4.12',
            'version' => '7.4.12.0',
            'reference' => '5cefb712a25f320579615ba9e1942abaeade7dff',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/mailer',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/mime' => array(
            'pretty_version' => 'v7.4.13',
            'version' => '7.4.13.0',
            'reference' => 'a845722765c4f6b2ce88beaf4f4479975b186770',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/mime',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/options-resolver' => array(
            'pretty_version' => 'v7.4.8',
            'version' => '7.4.8.0',
            'reference' => '2888fcdc4dc2fd5f7c7397be78631e8af12e02b4',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/options-resolver',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-ctype' => array(
            'pretty_version' => 'v1.37.0',
            'version' => '1.37.0.0',
            'reference' => '141046a8f9477948ff284fa65be2095baafb94f2',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-ctype',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-intl-grapheme' => array(
            'pretty_version' => 'v1.37.0',
            'version' => '1.37.0.0',
            'reference' => '4864388bfbd3001ce88e234fab652acd91fdc57e',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-intl-grapheme',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-intl-idn' => array(
            'pretty_version' => 'v1.38.1',
            'version' => '1.38.1.0',
            'reference' => 'dc21118016c039a66235cf93d96b435ffb282412',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-intl-idn',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-intl-normalizer' => array(
            'pretty_version' => 'v1.38.0',
            'version' => '1.38.0.0',
            'reference' => '2d446c214bdbe5b71bde5011b060a05fece3ae6b',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-intl-normalizer',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-mbstring' => array(
            'pretty_version' => 'v1.38.1',
            'version' => '1.38.1.0',
            'reference' => '14c5439eec4ccff081ac14eca2dc57feb2a66d92',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-mbstring',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-php80' => array(
            'pretty_version' => 'v1.37.0',
            'version' => '1.37.0.0',
            'reference' => 'dfb55726c3a76ea3b6459fcfda1ec2d80a682411',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-php80',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-php83' => array(
            'pretty_version' => 'v1.38.1',
            'version' => '1.38.1.0',
            'reference' => '8339098cae28673c15cce00d80734af0453054e2',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-php83',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-php84' => array(
            'pretty_version' => 'v1.37.0',
            'version' => '1.37.0.0',
            'reference' => '88486db2c389b290bf87ff1de7ebc1e13e42bb06',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-php84',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-php85' => array(
            'pretty_version' => 'v1.38.1',
            'version' => '1.38.1.0',
            'reference' => 'ba2ba04f3352cfa2dcbbcb90aee13ed967f505b1',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-php85',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-uuid' => array(
            'pretty_version' => 'v1.37.0',
            'version' => '1.37.0.0',
            'reference' => '26dfec253c4cf3e51b541b52ddf7e42cb0908e94',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-uuid',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/process' => array(
            'pretty_version' => 'v7.4.11',
            'version' => '7.4.11.0',
            'reference' => 'd9593c9efa40499eb078b81144de42cbc28a31f0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/process',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/routing' => array(
            'pretty_version' => 'v7.4.13',
            'version' => '7.4.13.0',
            'reference' => '3a162171bb008e5e0f15dce6581373a4c0e8390d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/routing',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/service-contracts' => array(
            'pretty_version' => 'v3.7.0',
            'version' => '3.7.0.0',
            'reference' => 'd25d82433a80eba6aa0e6c24b61d7370d99e444a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/service-contracts',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/string' => array(
            'pretty_version' => 'v7.4.11',
            'version' => '7.4.11.0',
            'reference' => '965f7306a43383d02c6aca1e3f3bd2f0ea5dee15',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/string',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/translation' => array(
            'pretty_version' => 'v7.4.10',
            'version' => '7.4.10.0',
            'reference' => 'ada7578c30dd5feaa8259cff3e885069ea81ddde',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/translation',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/translation-contracts' => array(
            'pretty_version' => 'v3.7.0',
            'version' => '3.7.0.0',
            'reference' => '0ab302977a952b42fd51475c4ebac81f8da0a95d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/translation-contracts',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/translation-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '2.3|3.0',
            ),
        ),
        'symfony/uid' => array(
            'pretty_version' => 'v7.4.9',
            'version' => '7.4.9.0',
            'reference' => '2676b524340abcfe4d6151ec698463cebafee439',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/uid',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/var-dumper' => array(
            'pretty_version' => 'v7.4.8',
            'version' => '7.4.8.0',
            'reference' => '9510c3966f749a1d1ff0059e1eabef6cc621e7fd',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/var-dumper',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/yaml' => array(
            'pretty_version' => 'v7.4.12',
            'version' => '7.4.12.0',
            'reference' => '8b6952b56ca6417f25f7a65758cadd0ce02edc51',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/yaml',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'tightenco/ziggy' => array(
            'pretty_version' => 'v2.6.2',
            'version' => '2.6.2.0',
            'reference' => '8a0b645921623f77dceaf543d61ecd51a391d96e',
            'type' => 'library',
            'install_path' => __DIR__ . '/../tightenco/ziggy',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'tijsverkoyen/css-to-inline-styles' => array(
            'pretty_version' => 'v2.4.0',
            'version' => '2.4.0.0',
            'reference' => 'f0292ccf0ec75843d65027214426b6b163b48b41',
            'type' => 'library',
            'install_path' => __DIR__ . '/../tijsverkoyen/css-to-inline-styles',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'videlalvaro/php-amqplib' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v3.7.4',
            ),
        ),
        'vlucas/phpdotenv' => array(
            'pretty_version' => 'v5.6.3',
            'version' => '5.6.3.0',
            'reference' => '955e7815d677a3eaa7075231212f2110983adecc',
            'type' => 'library',
            'install_path' => __DIR__ . '/../vlucas/phpdotenv',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'voku/portable-ascii' => array(
            'pretty_version' => '2.1.1',
            'version' => '2.1.1.0',
            'reference' => '8e1051fe39379367aecf014f41744ce7539a856f',
            'type' => 'library',
            'install_path' => __DIR__ . '/../voku/portable-ascii',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'web-token/jwt-library' => array(
            'pretty_version' => '4.1.6',
            'version' => '4.1.6.0',
            'reference' => 'e8ab00927a3856f3f0c8218226382cd6a58928a1',
            'type' => 'library',
            'install_path' => __DIR__ . '/../web-token/jwt-library',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'webmozart/assert' => array(
            'pretty_version' => '1.12.1',
            'version' => '1.12.1.0',
            'reference' => '9be6926d8b485f55b9229203f962b51ed377ba68',
            'type' => 'library',
            'install_path' => __DIR__ . '/../webmozart/assert',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
    ),
);
