<?php

return [
    'alerts' => [
        'title' => 'Alarmi',
    ],
    'alertlog' => [
        'title' => 'Dnevnik alarma',
    ],
    'alertlog-stats' => [
        'title' => 'Dnevnik statusa alarma',
    ],
    'availability-map' => [
        'title' => 'Mapa dostupnosti',
    ],
    'component-status' => [
        'title' => 'Statusi uređaja brojčano',
    ],
    'device-summary-horiz' => [
        'title' => 'Sumirani uređaji horizontalno',
    ],
    'device-summary-vert' => [
        'title' => 'Sumirani uređaji vertikalno',
    ],
    'device-types' => [
        'title' => 'Tipovi uređaja',
    ],
    'eventlog' => [
        'title' => 'Dnevnik događaja',
    ],
    'generic-graph' => [
        'title' => 'Grafikon',
    ],
    'generic-image' => [
        'title' => 'Zasebna slika',
    ],
    'globe' => [
        'title' => 'Globus',
    ],
    'graylog' => [
        'title' => 'Graylog',
    ],
    'notes' => [
        'title' => 'Zabeleške',
    ],
    'server-stats' => [
        'title' => 'Status servera',
    ],
    'syslog' => [
        'title' => 'Syslog zapisi',
    ],
    'top-devices' => [
        'title' => 'Top uređaji',
    ],
    'top-errors' => [
        'title' => 'Najviše grešaka',
    ],
    'top-interfaces' => [
        'title' => 'Top interfejsi',
    ],
    'worldmap' => [
        'title' => 'Mapa sveta',
    ],
];
