<?php

return [
    'title' => 'სისტემის ჟურნალი',
    'severity' => [
        '0' => 'სასწრაფო',
        '1' => 'განგაში',
        '2' => 'კრიტიკული',
        '3' => 'შეცდომა',
        '4' => 'გაფრთხილება',
        '5' => 'შეტყობინება',
        '6' => 'საინფორმაციო',
        '7' => 'გამართვა',
    ],
    'facility' => [
        '0' => 'ბირთვის შეტყობინებები',
        '1' => 'მომხმარებლის დონის შეტყობინებები',
        '2' => 'ელფოსტის სისტემა',
        '3' => 'სისტემური დემონები',
        '4' => 'უსაფრთხოების/ავტორიზაციის შეტყობინებები',
        '5' => 'Syslogd-ის მიერ შექმნილი შეტყობინებები',
        '6' => 'ხაზოვანი პრინტერის ქვესისტემა',
        '7' => 'ქსელური სიახლეების ქვესისტემა',
        '8' => 'ქვესისტმეაUUCP',
        '9' => 'საათის დემონი',
        '10' => 'უსაფრთხოების/ავტორიზაციის შეტყობინებები',
        '11' => 'FTP-ის დემონი',
        '12' => 'NTP-ის ქვესისტემა',
        '13' => 'ჟურნალის აუდიტი',
        '14' => 'ჟურნალის განგაში',
        '15' => 'საათის დემონი (შენიშვნა 2)',
        '16' => 'ლოკალური გამოყენება 0  (local0)',
        '17' => 'ლოკალური გამოყენება 1  (local1)',
        '18' => 'ლოკალური გამოყენება 2  (local2)',
        '19' => 'ლოკალური გამოყენება 3  (local3)',
        '20' => 'ლოკალური გამოყენება 4  (local4)',
        '21' => 'ლოკალური გამოყენება 5  (local5)',
        '22' => 'ლოკალური გამოყენება 6  (local6)',
        '23' => 'ლოკალური გამოყენება 7  (local7)',
    ],
];
