<?php

return [
    'settings_page' => ':plugin: Configuración',
    'admin_page' => 'Administración de plugins',
    'admin_title' => 'Plugins del sistema',
    'errors' => [
        'not_exist' => 'El plugin :plugin no existe.',
        'disabled' => 'El plugin :plugin está deshabilitado.',
        'view_missing' => 'Vista faltante.',
    ],
];
