<?php

$unit_text = 'per second';

$stats_list = [
    'thp_fault_alloc' => [
        'stat' => 'thp_fault_alloc',
        'descr' => 'thp_fault_alloc',
    ],
    'thp_collapse_alloc' => [
        'stat' => 'thp_collapse_alloc',
        'descr' => 'thp_collapse_alloc',
    ],
    'thp_swpout' => [
        'stat' => 'thp_swpout',
        'descr' => 'thp_swpout',
    ],
    'thp_swpout_fallback' => [
        'stat' => 'thp_swpout_fallback',
        'descr' => 'thp_swpout_fallback',
    ],
];

require 'oslv_monitor-common.inc.php';
