<?php

return [
    'attributes' => [
        'features' => 'Recursos do Sistema Operacional',
        'hardware' => 'Hardware',
        'icon' => 'Ícone',
        'ip' => 'IP',
        'location' => 'Localização',
        'os' => 'Sistema Operacional do Dispositivo',
        'serial' => 'Serial',
        'sysName' => 'sysName',
        'version' => 'Versão do Sistema Operacional',
        'type' => 'Tipo de dispositivo',
    ],
];
