<?php

return [

    /*
    |--------------------------------------------------------------------------
    | Языковые строки для постраничной навигации
    |--------------------------------------------------------------------------
    |
    | Следующие языковые строки используются библиотекой пагинации для создания
    | простых ссылок на постраничную навигацию. Вы можете изменить их на что угодно,
    | чтобы настроить ваши представления в соответствии с вашим приложением.
    |
    */

    'previous' => '&laquo; Назад',
    'next' => 'Вперед &raquo;',

];
