<?php

$unit_text = 'seconds';

$stats_list = [
    'cpu-time' => [
        'stat' => 'cpu-time',
        'descr' => 'CPU Time',
    ],
    'system-time' => [
        'stat' => 'system-time',
        'descr' => 'System Time',
    ],
    'user-time' => [
        'stat' => 'user-time',
        'descr' => 'User Time',
    ],
];

require 'poudriere-common.inc.php';
