<?php

return [

    /*
    |--------------------------------------------------------------------------
    | Wireless Sensors Language Lines
    |--------------------------------------------------------------------------
    |
    | The following language lines are used to translate names and units of wireless sensors
    |
    */

    'title' => 'უსადენო',
    'ap-count' => [
        'short' => 'AP-ები',
        'long' => 'AP-ების რაოდენობა',
        'unit' => '',
    ],
    'clients' => [
        'short' => 'კლიენტები',
        'long' => 'კლიენტების რაოდენობა',
        'unit' => '',
    ],
    'capacity' => [
        'short' => 'მოცულობა',
        'long' => 'მოცულობა',
        'unit' => '%',
    ],
    'ccq' => [
        'short' => 'კკხ',
        'long' => 'კლიენტის კავშირის ხარისხი',
        'unit' => '%',
    ],
    'errors' => [
        'short' => 'შეცდომები',
        'long' => 'შეცდომების რაოდენობა',
        'unit' => '',
    ],
    'error-ratio' => [
        'short' => 'შეცდომების შეფარდება',
        'long' => 'ბიტების/პაკეტების შეცდომის შეფარდება',
        'unit' => '%',
    ],
    'error-rate' => [
        'short' => 'ბშს',
        'long' => 'ბიტური შეცდომის სიხშირე',
        'unit' => 'ბ/წმ',
    ],
    'frequency' => [
        'short' => 'სიხშირე',
        'long' => 'სიხშირე',
        'unit' => 'მჰც',
    ],
    'distance' => [
        'short' => 'მანძილი',
        'long' => 'მანძილი',
        'unit' => 'მ',
    ],
    'mse' => [
        'short' => 'MSE',
        'long' => 'საშუალო კვადრატული შეცდომა',
        'unit' => 'დბ',
    ],
    'noise-floor' => [
        'short' => 'ხმის დონე',
        'long' => 'ხმის დონე',
        'unit' => 'დბმ',
    ],
    'power' => [
        'short' => 'სიმძლავრე/სიგნალი',
        'long' => 'TX/RX სიმძლავრე ანსიგნალი',
        'unit' => 'დბმ',
    ],
    'quality' => [
        'short' => 'ხარისხი',
        'long' => 'ხარისხი',
        'unit' => '%',
    ],
    'rate' => [
        'short' => 'სიჩქარე',
        'long' => 'TX/RX სიჩქარე',
        'unit' => 'ბ/წმ',
    ],
    'rssi' => [
        'short' => 'RSSI',
        'long' => 'მიღებული სიგნალის სიმძლავრის მაჩვენებელი',
        'unit' => 'დბმ',
    ],
    'snr' => [
        'short' => 'SNR',
        'long' => 'სიგნალის ფარდობა ხმაურთან',
        'unit' => 'dB',
    ],
    'sinr' => [
        'short' => 'SINR',
        'long' => ' სიგნალის ფარდობა ინტერფერენციასთან პლუს ხმაური',
        'unit' => 'dB',
    ],
    'rsrq' => [
        'short' => 'RSRQ',
        'long' => 'მიმართვითი სიგნალის მიღების ხარისხი',
        'unit' => 'dB',
    ],
    'rsrp' => [
        'short' => 'RSRP',
        'long' => 'მიმართვითი სიგნალის მიღების სიმძლავრე',
        'unit' => 'dBm',
    ],
    'ssr' => [
        'short' => 'SSR',
        'long' => 'სიგნალის სიმძლავრის ფარდობა',
        'unit' => 'dB',
    ],
    'utilization' => [
        'short' => 'გამოყენება',
        'long' => 'გამოყენება',
        'unit' => '%',
    ],
    'xpi' => [
        'short' => 'XPI',
        'long' => 'ჯვარედინი პოლარული ინტერფერენცია',
        'unit' => 'dB',
    ],
    'cell' => [
        'short' => 'უჯრედი',
        'long' => 'უჯრედი',
        'unit' => '',
    ],
    'channel' => [
        'short' => 'არხი',
        'long' => 'არხი',
        'unit' => '',
    ],
];
