<?php

$unit_text = 'bytes';

$stats_list = [
    'zswap' => [
        'stat' => 'zswap',
        'descr' => 'zswap size',
    ],
    'zswapped' => [
        'stat' => 'zswapped',
        'descr' => 'zswapped size',
    ],
];

require 'oslv_monitor-common.inc.php';
