<?php

return [
    'attributes' => [
        'features' => '操作系统特性',
        'hardware' => '硬件',
        'icon' => '图标',
        'ip' => 'IP地址',
        'location' => '位置',
        'os' => '设备操作系统',
        'serial' => '序列号',
        'sysName' => '系统名称',
        'version' => '操作系统版本',
        'type' => '设备类型',
    ],
];
