<?php

return [
    'notification-subscription-status' => [
        'no-support' => 'Этот браузер не поддерживает уведомления',
        'no-transport' => 'Чтобы включить уведомления в браузере, должен быть настроен транспорт уведомлений, ссылающийся на этого пользователя',
        'enabled' => 'Уведомления включены для этого браузера',
        'disabled' => 'Уведомления отключены для этого браузера',
        'enable' => 'Включить',
        'disable' => 'Отключить',
    ],
];
