<?php

return [
    'alerts' => [
        'title' => 'Alertas',
    ],
    'alertlog' => [
        'title' => 'Historial de alertas',
    ],
    'alertlog-stats' => [
        'title' => 'Estadísticas del historial de alertas',
    ],
    'alert-map' => [
        'title' => 'Mapa de alertas',
    ],
    'availability-map' => [
        'title' => 'Mapa de disponibilidad',
    ],
    'component-status' => [
        'title' => 'Estado de componentes',
    ],
    'custom-map' => [
        'title' => 'Mapa personalizado',
    ],
    'device-summary-horiz' => [
        'title' => 'Resumen de dispositivos horizontal',
    ],
    'device-summary-vert' => [
        'title' => 'Resumen de dispositivos vertical',
    ],
    'device-types' => [
        'title' => 'Tipos de dispositivos',
    ],
    'eventlog' => [
        'title' => 'Registro de eventos',
    ],
    'generic-graph' => [
        'title' => 'Gráfica',
    ],
    'generic-image' => [
        'title' => 'Imágenes externas',
    ],
    'globe' => [
        'title' => 'Mapa global',
    ],
    'graylog' => [
        'title' => 'Graylog',
    ],
    'notes' => [
        'title' => 'Notas',
    ],
    'placeholder' => [
        'title' => 'Marcador de posición',
    ],
    'server-stats' => [
        'title' => 'Estadísticas del servidor',
    ],
    'syslog' => [
        'title' => 'Syslog',
    ],
    'top-devices' => [
        'title' => 'Principales dispositivos',
    ],
    'top-errors' => [
        'title' => 'Principales errores',
    ],
    'top-interfaces' => [
        'title' => 'Principales interfaces',
    ],
    'world-map' => [
        'title' => 'Mapa mundial',
    ],
];
