<?php

return [
    'notification-subscription-status' => [
        'no-support' => 'Este navegador não suporta notificações',
        'no-transport' => 'Para habilitar notificações no navegador, deve haver um transporte de alerta referenciando este usuário',
        'enabled' => 'Notificações habilitadas para este navegador',
        'disabled' => 'Notificações desabilitadas para este navegador',
        'enable' => 'Habilitar',
        'disable' => 'Desabilitar',
    ],
];
