<?php

return [
    'alerts' => [
        'title' => '警報',
    ],
    'alertlog' => [
        'title' => '警報歷程',
    ],
    'alertlog-stats' => [
        'title' => '警報歷程統計',
    ],
    'availability-map' => [
        'title' => '可用性地圖',
    ],
    'component-status' => [
        'title' => '元件狀態',
    ],
    'custom-map' => [
        'title' => '自訂地圖',
    ],
    'device-summary-horiz' => [
        'title' => '橫式裝置概觀',
    ],
    'device-summary-vert' => [
        'title' => '直式裝置概觀',
    ],
    'device-types' => [
        'title' => '裝置類型',
    ],
    'eventlog' => [
        'title' => 'Eventlog',
    ],
    'generic-graph' => [
        'title' => '圖表',
    ],
    'generic-image' => [
        'title' => '外部影像',
    ],
    'globe' => [
        'title' => '全域地圖',
    ],
    'graylog' => [
        'title' => 'Graylog',
    ],
    'notes' => [
        'title' => '備註',
    ],
    'placeholder' => [
        'title' => '提示文字',
    ],
    'server-stats' => [
        'title' => '伺服器統計數據',
    ],
    'syslog' => [
        'title' => 'Syslog',
    ],
    'top-devices' => [
        'title' => '流量排行榜',
    ],
    'top-errors' => [
        'title' => '錯誤排行榜',
    ],
    'top-interfaces' => [
        'title' => '網路介面排行榜',
    ],
    'world-map' => [
        'title' => '世界地圖',
    ],
];
