<?php

return [
    'oui' => [
        'title' => 'MAC OUI 製造商查詢',
        'lookup' => 'OUI 查詢',
        'prompt' => '請輸入要查詢的 MAC 位址：',
        'no_db' => '目前安裝環境沒有任何廠商資料。此資料應由每日程序自動擷取，您可嘗試手動執行：:command。',
        'not_enabled' => '您必須先設定 :setting 才能使用此功能。',
    ],
];
