<?php

/**
 * ValidationGroup.php
 *
 * An interface for validation groups, this will be run by Validator.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * @link       https://www.librenms.org
 *
 * @copyright  2017 Tony Murray
 * @author     Tony Murray <murraytony@gmail.com>
 */

namespace LibreNMS\Interfaces;

use LibreNMS\Validator;

interface ValidationGroup
{
    /**
     * Validate this module.
     * To return ValidationResults, call ok, warn, fail, or result methods on the $validator
     */
    public function validate(Validator $validator): void;

    /**
     * Returns if this test should be run by default or not.
     */
    public function isDefault(): bool;

    /**
     * Returns true if this group has been run
     */
    public function isCompleted(): bool;

    /**
     * Mark this group as completed
     */
    public function markCompleted(): void;
}
