<?php

$unit_text = 'Count';

$stats_list = [
    'refer' => [
        'stat' => 'refer',
        'descr' => 'Refer Present',
    ],
    'no_refer' => [
        'stat' => 'no_refer',
        'descr' => 'Refer!Present',
    ],
];

require 'http_access_log_combined-common.inc.php';
