<?php

return [
    'notification-subscription-status' => [
        'no-support' => 'Este navegador no soporta notificaciones',
        'no-transport' => 'Para habilitar notificaciones del navegador, debe existir un transporte de alerta referenciando a este usuario',
        'enabled' => 'Notificaciones habilitadas para este navegador',
        'disabled' => 'Notificaciones deshabilitadas para este navegador',
        'enable' => 'Habilitar',
        'disable' => 'Deshabilitar',
    ],
    'maintenance-mode' => [
        'button' => [
            'maintenance_mode' => 'Modo mantenimiento',
            'device_under_maintenance' => 'Dispositivo en mantenimiento',
        ],
        'titles' => [
            'device_maintenance' => 'Mantenimiento del dispositivo',
            'end_maintenance' => 'Finalizar mantenimiento',
        ],
        'confirm' => [
            'end_prompt' => '¿Está seguro que desea finalizar el mantenimiento de este dispositivo?',
        ],
        'form' => [
            'notes_label' => 'Notas:',
            'notes_placeholder' => 'Notas de mantenimiento',
            'duration_label' => 'Duración:',
            'behavior_label' => 'Comportamiento:',
            'start_maintenance' => 'Iniciar mantenimiento',
            'end_maintenance' => 'Finalizar mantenimiento',
        ],
        'errors' => [
            'enable' => 'Ocurrió un error al poner este dispositivo en modo mantenimiento',
            'disable' => 'Ocurrió un error al deshabilitar el modo mantenimiento',
        ],
    ],
];
