export default {
    "en": {
        "help": {
            "yes": "see <a href=\"mailto:mail@com\">",
            "no": "see <a href=\"{link}\">",
            "maybe": "It is a <strong>Test</strong> ok!"
        }
    }
}
